# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

# from math import fabs

from PySide6.QtCore import Qt, Signal, QSizeF, QRectF, QPointF, Slot, QUrl
from PySide6.QtGui import QBrush, QPen, QColor, QPainterPath, QFontMetricsF, QTextCursor
from PySide6.QtWidgets import QGraphicsTextItem, QStyle
from .. import shared


########################################################################
class Text(QGraphicsTextItem):
    Type = shared.TEXT_ITEM_TYPE

    # textItemFocusedOut = Signal(QGraphicsTextItem)
    textItemFocusedOut = Signal(object)

    # textItemSelectedChanged = Signal(QGraphicsTextItem)

    # ---------------------------------------------------------------------
    def __init__(self, cursorPos, yaziRengi, arkaPlanRengi, pen, font, rect=None, text=None, parent=None):
        super(Text, self).__init__(text, parent)

        self._kim = shared.kim(kac_basamak=16)

        self.setAcceptHoverEvents(True)

        self.doc = self.document()
        # self.doc.cursorPositionChanged.connect(self.doc_layout_changed)
        self.docLayout = self.doc.documentLayout()
        # self.docLayout.documentSizeChanged.connect(self.doc_layout_changed)

        # self.doc.setUndoRedoEnabled(True)

        self.doc.undoCommandAdded.connect(self.act_undo_eklendi)

        self.setFont(font)

        self.isTextOverflowed = False
        self._resizing = False

        if not rect:
            # self._rect = QRectF()
            self._rect = super(Text, self).boundingRect()
            # self._rect = QRectF(QPointF(0,0), self.doc.size())
            # self._rect = self.boundingRect()
        else:
            # rect.moveTo(0, 0)  # gerek yok cunku zaten 0,0 liyoruz scene den gondeririken.
            self._rect = rect
            self.setTextWidth(rect.size().width())

        self.docLayout.documentSizeChanged.connect(self.doc_layout_changed)

        # self.setPos(cursorPos + QPointF(1, 1))
        self.setPos(cursorPos)

        self._eskiRectBeforeResize = None
        self._eskiPosBeforeResize = None

        self.secili_nesne_kalem_kalinligi = 0
        self.handleSize = 10
        self.resizeHandleSize = QSizeF(self.handleSize, self.handleSize)
        self.update_resize_handles()

        # self.setCacheMode(Text.DeviceCoordinateCache)
        # self.setCacheMode(Text.ItemCoordinateCache)
        self.setCacheMode(Text.CacheMode.NoCache)
        self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable |
                      QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable |
                      QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable)

        # self.setTextInteractionFlags(Qt.TextEditable)
        self.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction)

        self.cursorPos = cursorPos

        self.hiza = Qt.AlignmentFlag.AlignLeft

        # fmt = QTextBlockFormat()
        # fmt.setAlignment(Qt.AlignCenter)
        cursor = self.textCursor()
        cursor.select(QTextCursor.SelectionType.Document)
        # cursor.mergeBlockFormat(fmt)
        # cursor.clearSelection()
        self.setTextCursor(cursor)

        self._brush = QBrush()
        # we dont use QPen in this text item.
        # self.cizgiKalinligi = 0
        # self.cizgiTipi = Qt.SolidLine
        # self.cizgiUcuTipi = Qt.RoundCap
        # self.cizgiBirlesimTipi = Qt.RoundJoin

        self.activeItemLineColor = shared.activeItemLineColor
        # self.setDefaultTextColor(cizgiRengi)
        self._pen = pen
        # self.cizgiRengi = QColor(Qt.black)
        self.cizgiRengi = pen.color()  # TODO: hemen asagidaki methodta yine set ediyor
        self.setCizgiRengi(self.cizgiRengi)  # also sets defaultTextColor
        self.yaziRengi = yaziRengi
        self.setYaziRengi(yaziRengi)
        self.setArkaPlanRengi(arkaPlanRengi)  # also sets self._brush

        self.cosmeticSelect = False
        self.isActiveItem = False
        self._isPinned = False
        self.ustGrup = None
        self._command = {}

        self.isPlainText = True
        self.oklar_dxdy_nokta = {}

        # self.zenginYaziBelirteci = QRect(0, 0, 10, 10)

        # self.cursor_eski_anchor = 0
        # self.cursor_eski_pozisyon = 0
        # self.doc.cursorPositionChanged.connect(self.cursor_pos_degisti)

    # # ---------------------------------------------------------------------
    # def cursor_pos_degisti(self, cursor):
    #     self.cursor_eski_pozisyon = cursor.position()
    #     self.cursor_eski_anchor = cursor.anchor()

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return Text.Type

    # ---------------------------------------------------------------------
    @property
    def isPinned(self):
        return self._isPinned

    # ---------------------------------------------------------------------
    @isPinned.setter
    def isPinned(self, value):
        if value:
            self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable
                          # | QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable
                          #  | QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable
                          )

        else:
            self.setFlags(QGraphicsTextItem.GraphicsItemFlag.ItemIsSelectable
                          | QGraphicsTextItem.GraphicsItemFlag.ItemIsMovable
                          | QGraphicsTextItem.GraphicsItemFlag.ItemIsFocusable)
        self._isPinned = value

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "pen": self._pen,
                      "font": self.font(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "isPlainText": self.isPlainText,
                      "yaziHiza": int(self.doc.defaultTextOption().alignment()),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        if self.isPlainText:
            properties["text"] = self.toPlainText()
        else:
            properties["html"] = self.toHtml()
        return properties

    # ---------------------------------------------------------------------
    @Slot()
    def act_undo_eklendi(self):
        self.scene().undoRedo.undoRedoBaglantisiYaziNesnesiDocuna(self.scene().undoStack,
                                                                  self.scene().tr("change text"),
                                                                  self.doc)

    # ---------------------------------------------------------------------
    def ok_ekle(self, ok, scenepPos, okunHangiNoktasi):

        # self.oklar_dxdy_nokta.append((ok, self.mapFromScene(scenePos)))
        sceneCenter = self.sceneCenter()
        dx = sceneCenter.x() - scenepPos.x()
        dy = sceneCenter.y() - scenepPos.y()

        # self.oklar_dxdy_nokta[ok._kim] = (dx, dy, nokta)
        self.oklar_dxdy_nokta[ok] = (dx, dy, okunHangiNoktasi)

        ok.baglanmis_nesneler[self._kim] = okunHangiNoktasi

    # ---------------------------------------------------------------------
    def ok_sil(self, ok):
        del self.oklar_dxdy_nokta[ok]
        del ok.baglanmis_nesneler[self._kim]

    # ---------------------------------------------------------------------
    def ok_guncelle(self):
        if self.oklar_dxdy_nokta:
            sceneCenter = self.sceneCenter()
            scx = sceneCenter.x()
            scy = sceneCenter.y()
            for ok, dxdy_nokta in self.oklar_dxdy_nokta.items():
                if dxdy_nokta[2] == 1:
                    ok.temp_prepend(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                elif dxdy_nokta[2] == 2:
                    ok.temp_append(QPointF(scx - dxdy_nokta[0], scy - dxdy_nokta[1]))
                if ok._text:
                    ok.update_painter_text_rect()

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def set_document_url(self, url):

        # # ifdef Q_WS_WIN
        # document()->setMetaInformation(QTextDocument::DocumentUrl, imagePath + "/" );
        # # else
        # document()->setMetaInformation(QTextDocument::DocumentUrl, "file:" + imagePath + "/");
        # # endif

        # self.doc.setMetaInformation(self.doc.DocumentUrl, "file:{}/images/".format(url))

        # http://doc.qt.io/qt-5/qtextdocument.html#baseUrl-prop

        # TODO: bu aslinda boyle degil, url sonda / ile gelmiyor
        # url already comes with a "/" suffix so we use double slash "file://{url}"
        # self.doc.setBaseUrl(QUrl("file://{}/images-html/".format(url)))
        self.doc.setBaseUrl(QUrl("file://{}/".format(url)))
        # self.doc.setBaseUrl(QUrl(url))
        # print(self.doc.metaInformation(self.doc.DocumentUrl))

    # ---------------------------------------------------------------------
    def get_document_url(self):
        return self.doc.baseUrl()

    # ---------------------------------------------------------------------
    @Slot(QSizeF)
    def doc_layout_changed(self, size):
        # self._rect.setSize(self.doc.size())
        if size.width() > self._rect.size().width() or size.height() > self._rect.size().height():
            self._rect.setSize(size)

        self.update_resize_handles()

    # ---------------------------------------------------------------------
    def update_resize_handles(self):
        self.prepareGeometryChange()

        if self.scene():
            resizeHandleSize = self.resizeHandleSize / self.scene().views()[0].transform().m11()
        else:
            resizeHandleSize = self.resizeHandleSize

        self.topLeftHandle = QRectF(self._rect.topLeft(), resizeHandleSize)
        self.topRightHandle = QRectF(self._rect.topRight(), resizeHandleSize)
        self.bottomRightHandle = QRectF(self._rect.bottomRight(), resizeHandleSize)
        self.bottomLeftHandle = QRectF(self._rect.bottomLeft(), resizeHandleSize)

        # self.topLeftHandle.moveTopLeft(self.boundingRect().topLeft())
        self.topRightHandle.moveTopRight(self.boundingRect().topRight())
        self.bottomRightHandle.moveBottomRight(self.boundingRect().bottomRight())
        self.bottomLeftHandle.moveBottomLeft(self.boundingRect().bottomLeft())

    # ---------------------------------------------------------------------
    def sceneCenter(self):
        if self.parentItem():
            return self.mapToParent(self.boundingRect().center())
        return self.mapToScene(self.boundingRect().center())

    # ---------------------------------------------------------------------
    def sceneWidth(self):
        return self.sceneRight() - self.sceneLeft()

    # ---------------------------------------------------------------------
    def sceneHeight(self):
        return self.sceneBottom() - self.sceneTop()

    # ---------------------------------------------------------------------
    def sceneRight(self):
        return max(self.sceneBoundingRect().topRight().x(),
                   self.sceneBoundingRect().bottomRight().x())

    # ---------------------------------------------------------------------
    def sceneLeft(self):
        return min(self.sceneBoundingRect().topLeft().x(),
                   self.sceneBoundingRect().bottomLeft().x())

    # ---------------------------------------------------------------------
    def sceneTop(self):
        return min(self.sceneBoundingRect().topLeft().y(),
                   self.sceneBoundingRect().topRight().y())

    # ---------------------------------------------------------------------
    def sceneBottom(self):
        return max(self.sceneBoundingRect().bottomRight().y(),
                   self.sceneBoundingRect().bottomLeft().y())

    # ---------------------------------------------------------------------
    def setSceneLeft(self, left):
        self.setX(left + self.scenePos().x() - self.sceneLeft())

    # ---------------------------------------------------------------------
    def setSceneRight(self, right):
        self.setX(right + self.scenePos().x() - self.sceneRight())

    # ---------------------------------------------------------------------
    def setSceneTop(self, top):
        self.setY(top + self.scenePos().y() - self.sceneTop())

    # ---------------------------------------------------------------------
    def setSceneBottom(self, bottom):
        self.setY(bottom + self.scenePos().y() - self.sceneBottom())

    # ---------------------------------------------------------------------
    def setHorizontalCenter(self, hcenter):
        self.setY(hcenter + self.scenePos().y() - self.sceneCenter().y())

    # ---------------------------------------------------------------------
    def setVerticalCenter(self, vcenter):
        self.setX(vcenter + self.scenePos().x() - self.sceneCenter().x())

    # ---------------------------------------------------------------------
    def flipHorizontal(self, mposx):
        ipos = self.scenePos()
        rc = self.mapToScene(self.boundingRect().center())
        diff = mposx - rc.x()

        ipos.setX(ipos.x() + 2 * diff)
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))
        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipHorizontal(self.sceneCenter().x())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def flipVertical(self, mposy):
        ipos = self.scenePos()
        rc = self.mapToScene(self.boundingRect().center())
        diff = mposy - rc.y()

        ipos.setY(ipos.y() + 2 * diff)
        if self.parentItem():
            # ipos = (ipos - self.parentItem().scenePos())
            ipos = (self.parentItem().mapFromScene(ipos))
        self.setPos(ipos)
        if self.rotation():
            self.rotateWithOffset(360 - self.rotation())

        eskiRot = self.rotation()
        if eskiRot:
            self.rotateWithOffset(0)
        for c in self.childItems():
            c.flipVertical(self.sceneCenter().y())
        if eskiRot:
            self.rotateWithOffset(eskiRot)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_yazi_sag_menu_about_to_show(self)
        self.scene().parent().yaziSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def setText(self, text):
        self.setPlainText(text)

    # ---------------------------------------------------------------------
    def text(self):
        return self.toPlainText()

    # ---------------------------------------------------------------------
    def setCommand(self, title, command):
        self._command["title"] = title
        self._command["command"] = command

    # ---------------------------------------------------------------------
    def command(self):
        return self._command

    # ---------------------------------------------------------------------
    def setFont(self, font, yaziTipiMiDegisti=True):
        if yaziTipiMiDegisti:
            font.setBold(self.font().bold())
            font.setItalic(self.font().italic())
            font.setUnderline(self.font().underline())
            font.setStrikeOut(self.font().strikeOut())
            font.setOverline(self.font().overline())
            # font.setPointSizeF(self.font().pointSizeF())
        super(Text, self).setFont(font)
        if self.parentItem():
            self.yazi_kutusunu_daralt()

    # ---------------------------------------------------------------------
    def setFontPointSizeF(self, fontPointSizeF):
        font = self.font()
        font.setPointSizeF(fontPointSizeF)
        self.setFont(font, yaziTipiMiDegisti=False)
        self.update()

    # ---------------------------------------------------------------------
    def fontPointSizeF(self):
        return self.font().pointSizeF()

    # ---------------------------------------------------------------------
    def ver_yazi_hizasi(self):
        return self.doc.defaultTextOption().alignment()

    # ---------------------------------------------------------------------
    def kur_yazi_hizasi(self, hiza):
        option = self.doc.defaultTextOption()
        option.setAlignment(hiza)
        self.doc.setDefaultTextOption(option)

    # ---------------------------------------------------------------------
    def ver_karakter_bicimi(self):
        # return self._font.bold | self._font.italic | self._font.under | self._font.strikeout
        font = self.font()
        return {"b": font.bold(),
                "i": font.italic(),
                "u": font.underline(),
                "s": font.strikeOut(),
                "o": font.overline(),
                }

    # ---------------------------------------------------------------------
    def kur_karakter_bicimi(self, sozluk):
        font = self.font()
        font.setBold(sozluk["b"])
        font.setItalic(sozluk["i"])
        font.setUnderline(sozluk["u"])
        font.setStrikeOut(sozluk["s"])
        font.setOverline(sozluk["o"])
        self.setFont(font, yaziTipiMiDegisti=False)
        # self.update()

    # ---------------------------------------------------------------------
    def setPen(self, pen):

        if self._pen == pen:
            return
        self.prepareGeometryChange()
        self._pen = pen
        # self._boundingRect = QRectF()
        self.update(self.boundingRect())

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen

    # ---------------------------------------------------------------------
    def setBrush(self, brush):

        if self._brush == brush:
            return
        self._brush = brush
        self.update()

    # ---------------------------------------------------------------------
    def brush(self):
        return self._brush

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):

        _selectionLineBgColor = QColor.fromHsv(col.hue(),
                                               0,
                                               20 if col.value() > 127 else 250)

        _activeItemLineColor = QColor(self.activeItemLineColor)
        if col.hue() > 300 or col.hue() < 30:
            _activeItemLineColor.setHsv((col.hue() + 150) % 360,
                                        self.activeItemLineColor.saturation(),
                                        self.activeItemLineColor.value())
        self._pen.setColor(col)
        # self.setDefaultTextColor(col)
        self.cizgiRengi = col

        self.selectionPenBottom = QPen(_selectionLineBgColor,
                                       self.secili_nesne_kalem_kalinligi,
                                       Qt.PenStyle.DashLine,
                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)
        self.selectionPenBottomIfAlsoActiveItem = QPen(_activeItemLineColor,
                                                       self.secili_nesne_kalem_kalinligi,
                                                       Qt.PenStyle.DashLine,
                                                       Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin)

        self.update()

    # ---------------------------------------------------------------------
    def setYaziRengi(self, col):

        self.yaziRengi = col
        # self.textPen = QPen(QColor().fromRgb(col.red(), col.green(), col.blue(), col.alpha()))
        self.setDefaultTextColor(col)
        self.update()

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.update()

    # ---------------------------------------------------------------------
    def setArkaPlanRengi(self, col):
        self.arkaPlanRengi = col
        self.setBrush(QBrush(col))

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):

        if change == QGraphicsTextItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().text_item_selected(self)
            else:
                self.scene().parent().text_item_deselected(self)

        # return QGraphicsRectItem.itemChange(self, change, value)
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    def focusInEvent(self, event):

        if self.scene():
            if self.doc.availableUndoSteps():
                self.scene().parent().log(
                    self.tr("{} undo(s), {} redo(s) available for items' text.").format(self.doc.availableUndoSteps(),
                                                                                        self.doc.availableRedoSteps()),
                    4500,
                    toStatusBarOnly=True)

        super(Text, self).focusInEvent(event)

    # ---------------------------------------------------------------------
    def focusOutEvent(self, event):

        self.textItemFocusedOut.emit(self)
        if self.scene():
            self.scene().parent()._statusBar.clearMessage()

        # self.setTextInteractionFlags(Qt.TextSelectableByMouse)
        # self.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction)
        # self.clear_selection()
        if self.textCursor().hasSelection():
            return QGraphicsTextItem.focusOutEvent(self, event)

        super(Text, self).focusOutEvent(event)

    # ---------------------------------------------------------------------
    def clear_selection(self):
        cursor = self.textCursor()
        # self.cursor_eski_pozisyon = cursor.position()
        # self.cursor_eski_anchor = cursor.anchor()
        cursor.clearSelection()
        self.setTextCursor(cursor)

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        # if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().parent().web_browser_ac(self.text())
        # elif event.modifiers() == Qt.KeyboardModifier.ControlModifier | Qt.KeyboardModifier.ShiftModifier:
        #     self.scene().parent().web_browser_ac("https://www.google.com/search?q=" + self.text())
        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            self.yazi_kutusunu_daralt()

        # elif event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
        #     self.setTextInteractionFlags(Qt.TextInteractionFlag.NoTextInteraction)
        #     self.clear_selection()

        else:
            # if self.textInteractionFlags() == Qt.NoTextInteraction:
            self.setTextInteractionFlags(Qt.TextInteractionFlag.TextEditorInteraction)
            self.setFocus()
            # self.clearFocus()
        super(Text, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(Text, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):

        # cursor = self.scene().parent().cursor()

        # if self.isSelected() and self.scene().aktifArac == self.scene().SecimAraci:
        if not self.ustGrup:
            if self.scene().aktifArac == self.scene().SecimAraci:
                if self.topLeftHandle.contains(event.pos()) or self.bottomRightHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeFDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeFDiagCursor, gecici_mi=True)
                elif self.topRightHandle.contains(event.pos()) or self.bottomLeftHandle.contains(event.pos()):
                    self.scene().parent().setCursor(Qt.CursorShape.SizeBDiagCursor, gecici_mi=True)
                    # self.setCursor(Qt.SizeBDiagCursor, gecici_mi=True)
                else:
                    self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Text, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # if self.isSelected():
        # self.setCursor(self.saved_cursor)
        if not self.ustGrup:
            self.scene().parent().setCursor(self.scene().parent().imlec_arac, gecici_mi=True)

        super(Text, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):

        if self.ustGrup:
            return QGraphicsTextItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:
        if toplam == ctrlShift:
            # self.scaleItem(event.delta())
            self.scaleItemWithFontSizeF(event.delta())
            # self.scaleItem(event.angleDelta().y())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())

        elif toplam == altShift:
            # self.changeImageItemTextBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemWithFontSizeF(event.delta())

        else:
            super(Text, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def scaleItemWithFontSizeF2(self, delta):
        self.changeFontSizeF(delta)

    # ---------------------------------------------------------------------
    def scaleItemWithFontSizeF(self, delta):

        scaleFactor = 1.1
        if delta < 0:
            scaleFactor = 1 / scaleFactor
            # scaleFactor = 0.91

        # self.setTransformOriginPoint(self.boundingRect().center())
        # fontPointSizeF = max(3, (self.fontPointSizeF() * scaleFactor))
        fontPointSizeF = self.fontPointSizeF() * scaleFactor

        # eskiTamCerceve = self.boundingRect() | self.childrenBoundingRect()
        # yeniTamCerceveSize = eskiTamCerceve.size() * scaleFactor

        self.scene().undoRedo.undoableScaleTextItemByResizing(self.scene().undoStack,
                                                              "_scale by resizing",
                                                              self, scaleFactor, fontPointSizeF
                                                              )

        self.yazi_kutusunu_daralt()

    # ---------------------------------------------------------------------
    def changeFontSizeF(self, delta):
        # font = self.font()
        size = self.fontPointSizeF()

        if delta > 0:
            # font.setPointSizeF(size + 1)
            size += 1

        else:
            if size > 3:
                # font.setPointSizeF(size - 1)
                size -= 1
            else:
                # undolari biriktermesin diye donuyoruz,
                # yoksa zaten ayni size de yeni bir undolu size komutu veriyor.
                return

        self.scene().undoRedo.undoableSetFontSizeF(self.scene().undoStack, "change text size", self, size)
        self.yazi_kutusunu_daralt()
        # TODO: child items baseden bak

    # ---------------------------------------------------------------------
    def rotateWithOffset(self, angle):
        cEski = self.sceneCenter()
        self.setRotation(angle)
        cYeni = self.sceneCenter()
        diff = cEski - cYeni
        self.moveBy(diff.x(), diff.y())

    # ---------------------------------------------------------------------
    def rotateItem(self, delta):

        # self.setTransformOriginPoint(self.boundingRect().center())
        if delta > 0:
            # self.setRotation(self.rotation() + 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() + 5)
        else:
            # self.setRotation(self.rotation() - 5)
            self.scene().undoRedo.undoableRotateWithOffset(self.scene().undoStack, "rotate", self, self.rotation() - 5)
        # self.update()

    # ---------------------------------------------------------------------
    def changeBackgroundColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, QColor(self.arkaPlanRengi))

        # self.setBrush(self.arkaPlanRengi)
        # self.update()
        # self.scene().undoableSetItemBackgroundColorAlpha("Change item's background alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' background alpha")
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "_change item's background alpha",
                                                                      self, col)
            for c in self.childItems():
                c.changeBackgroundColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetItemBackgroundColorAlpha(self.scene().undoStack,
                                                                      "change item's background alpha", self,
                                                                      col)

    # ---------------------------------------------------------------------
    def changeLineColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, self.defaultTextColor())

        # self.setDefaultTextColor(col)
        # self.scene().undoableSetLineOrTextColorAlpha("Change item's text alpha", self, col)
        if self.childItems():
            self.scene().undoStack.beginMacro("change items' line alpha")
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "_change item's line alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeLineColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetLineColorAlpha(self.scene().undoStack, "change item's line alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeTextColorAlpha(self, delta):

        col = shared.calculate_alpha(delta, self.yaziRengi)

        if self.childItems():
            self.scene().undoStack.beginMacro("change items' text alpha")
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "_change item's text alpha", self,
                                                            col)
            for c in self.childItems():
                c.changeTextColorAlpha(delta)
            self.scene().undoStack.endMacro()
        else:
            self.scene().undoRedo.undoableSetTextColorAlpha(self.scene().undoStack, "change item's text alpha", self,
                                                            col)

    # ---------------------------------------------------------------------
    def changeImageItemTextBackgroundColorAlpha(self, delta):
        # grup nesnesinden cagriliyor alt+shift
        if self.childItems():
            startedMacro = False
            for c in self.childItems():
                if c.type() == shared.IMAGE_ITEM_TYPE:
                    if not startedMacro:
                        self.scene().undoStack.beginMacro("change image items' text background alpha")
                        startedMacro = True
                    c.changeImageItemTextBackgroundColorAlpha(delta)
            if startedMacro:
                self.scene().undoStack.endMacro()

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):

        self._resizing = False  # we could use "self._resizingFrom = 0" instead, but self._resizing is more explicit.
        self._fixedResizePoint = None
        self._resizingFrom = None
        self._eskiRectBeforeResize = None

        self._eskiPosBeforeResize = self.pos()
        # self._eskiPosBeforeResize = self.scenePos()

        if self.topLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomRight()
            self._resizingFrom = 1
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.topRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.bottomLeft()
            self._resizingFrom = 2
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.bottomRightHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topLeft()
            self._resizingFrom = 3
            self._eskiRectBeforeResize = QRectF(self._rect)
        elif self.bottomLeftHandle.contains(event.pos()):
            self._resizing = True
            self._fixedResizePoint = self._rect.topRight()
            self._resizingFrom = 4
            self._eskiRectBeforeResize = QRectF(self._rect)

        super(Text, self).mousePressEvent(event)

        # TODO: bu kozmetik amacli, sadece release de olunca gecikmeli seciyormus hissi oluyor
        if self.isSelected() and self.childItems():
            self.scene().select_all_children_recursively(self, cosmeticSelect=True)

        # self.scene().deactivate_item(self)
        # self.scene().activate_item(self)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        if self._resizing:
            # sahneye ilk cizim kontrolu,(undoya eklemesin diye)
            # if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
            #     rect = self.boundingRect()
            #     self.setPos(self.mapToScene(rect.topLeft()))
            #     rect.moveTo(0, 0)
            #     self.scene().undoableResizeBaseItem("resize",
            #                                         self,
            #                                         # yeniRect=self._rect,
            #                                         yeniRect=rect,
            #                                         eskiRect=self._eskiRectBeforeResize,
            #                                         eskiPos=self._eskiPosBeforeResize)

            # TODO: bu satir eger cizim disari tasarsa, scene recti ona uygun ayarlamak icin
            # fakat eger cizim disari tasarsa evet uyarliyor ama bazen de çizilen itemin geometrisini degistiriyor.
            # o yuzden simdilik iptal.
            # self.scene().unite_with_scene_rect(self.sceneBoundingRect())

            # rect = self.boundingRect()
            rect = self._rect
            # self.setPos(self.mapToScene(rect.topLeft()))
            # rect.moveTo(0, 0)
            # self.doc.setTextWidth(rect.width())

            # undo icinde
            # self.update_resize_handles()

            self.scene().undoRedo.undoableResizeBaseItem(self.scene().undoStack,
                                                         "resize",
                                                         self,
                                                         # yeniRect=self._rect,
                                                         yeniRect=rect,
                                                         eskiRect=self._eskiRectBeforeResize,
                                                         eskiPos=self._eskiPosBeforeResize)

            # print(self.mapRectToScene(self._eskiRectBeforeResize))
            # self.scene().update(self.mapRectToScene(self._eskiRectBeforeResize))
            # self.scene().views()[0].updateScene([self.mapRectToScene(self._eskiRectBeforeResize)])
            self._resizing = False
            self.scene().unite_with_scene_rect(self.sceneBoundingRect())

        super(Text, self).mouseReleaseEvent(event)

        if self.childItems():
            if self.isSelected():
                self.scene().select_all_children_recursively(self, cosmeticSelect=False, topmostParent=True)
            else:
                self.scene().select_all_children_recursively(self, cosmeticSelect=False)

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        if self._resizing:

            px = self._fixedResizePoint.x()
            py = self._fixedResizePoint.y()

            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)
            # rect = QRectF(QPointF(0, 0), size)

            rect = QRectF(topLeft, bottomRight)
            # rect = rect.normalized()

            # self.setPos(self.mapToScene(topLeft))
            # self.setPos(self.mapFromScene(topLeft))
            # eger bu move to 0,0 koymazsak o zaman normal resize ediyoruz ama
            # sol taraftan resize edersek release edince text tepki veriyor.
            rect.moveTo(0, 0)
            self.setRect(rect)
            # self.setPos(self.mapToScene(rect.topLeft()))
            self.scene().parent().change_transform_box_values(self)

        # event.accept()
        else:
            super(Text, self).mouseMoveEvent(event)

        # super(BaseItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def yazi_kutusunu_daralt(self):
        # self._rect.setSize(QSizeF(self.doc.size()))
        r = QRectF(self._rect)
        # r.setSize(QSizeF(self.textWidth(), self.doc.size().height()))

        fm = QFontMetricsF(self.font())
        # yaziGenislik = fm.horizontalAdvance(self.text(), QTextOption())
        yaziGenislik = fm.boundingRect(r, 0, self.text()).size().width()
        # yaziYukseklik = fm.height()
        # self.doc.adjustSize()
        # self.doc.setTextWidth(yaziGenislik + 10)
        r.setWidth(yaziGenislik + 10)
        r.setHeight(self.doc.size().height())
        # r.setSize(QSizeF(self.doc.size()))
        self.setRect(r)

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return

        self.prepareGeometryChange()
        self._rect = rect
        # TODO: burda direkt self.settextwidth niye demedik ki
        self.doc.setTextWidth(rect.width())
        # self.doc.setPageSize(rect.size())
        self.update_resize_handles()
        # self.adjustSize()
        # self.scene().parent().change_transform_box_values(self)
        # self._boundingRect = QRectF()

        if self.doc.size().height() > rect.size().height():
            self.isTextOverflowed = True
        else:
            self.isTextOverflowed = False
        self.update()

    # ---------------------------------------------------------------------
    def rect(self):
        return self._rect

    # ---------------------------------------------------------------------
    def shape(self):

        # if (!dd->control)
        # return QPainterPath();
        #
        # QPainterPath path;
        # path.addRect(dd->boundingRect);
        # return path;

        # path = QPainterPath()
        # # path.addRect(self._rect)
        # if self._rect.isEmpty():
        #     return super(Text, self).shape()
        # else:
        #     path.addRect(self._rect)
        # # return self.qt_graphicsItem_shapeFromPath(path, self._pen)
        #     return path

        path = QPainterPath()
        path.addRect(self._rect)
        return path

    # ---------------------------------------------------------------------
    def boundingRectttt(self):
        if self._rect.isEmpty():
            print("empty")
            return super(Text, self).boundingRect()
            # self.setRect(super(Text, self).boundingRect())
        return self._rect

    # ---------------------------------------------------------------------
    def boundingRecttt(self):
        bRect = super(Text, self).boundingRect()
        if bRect.width() > self._rect.width():
            self._rect.setSize(QSizeF(bRect.width(), self._rect.height()))
        if bRect.height() > self._rect.height():
            self._rect.setSize(QSizeF(self._rect.width(), bRect.height()))
        return self._rect

    # ---------------------------------------------------------------------
    def boundingRect(self):
        return self._rect

    # void QGraphicsTextItemPrivate::_q_updateBoundingRect(const QSizeF &size)
    # {
    #     if (!control) return; // can't happen
    #     const QSizeF pageSize = control->document()->pageSize();
    #     // paged items have a constant (page) size
    #     if (size == boundingRect.size() || pageSize.height() != -1)
    #         return;
    #     qq->prepareGeometryChange();
    #     boundingRect.setSize(size);
    #     qq->update();
    # }

    # # ---------------------------------------------------------------------
    # def qt_graphicsItem_shapeFromPath(self, path, pen):
    #     # We unfortunately need this hack as QPainterPathStroker will set a width of 1.0
    #     # if we pass a value of 0.0 to QPainterPathStroker.setWidth()
    #     penWidthZero = 0.00000001
    #     if path == QPainterPath():
    #         return path
    #     ps = QPainterPathStroker()
    #     ps.setCapStyle(pen.capStyle())
    #
    #     if pen.widthF() <= 0:
    #         ps.setWidth(penWidthZero)
    #     else:
    #         ps.setWidth(pen.widthF())
    #     ps.setJoinStyle(pen.joinStyle())
    #     ps.setMiterLimit(pen.miterLimit())
    #     p = ps.createStroke(path)  # return type: QPainterPath
    #     p.addPath(path)
    #     return p

    # ---------------------------------------------------------------------
    #         inline QPointF controlOffset() const
    #         {
    #         return QPointF(0., pageNumber * control->document()->pageSize().height());}

    # ---------------------------------------------------------------------
    #     void
    #     QGraphicsTextItem::paint(QPainter * painter, const
    #     QStyleOptionGraphicsItem * option,
    #     QWidget * widget)
    #
    # {
    #     Q_UNUSED(widget);
    # if (dd->control) {
    # painter->save();
    # QRectF r = option->exposedRect;
    # painter->translate(-dd->controlOffset());
    # r.translate(dd->controlOffset());
    #
    # QTextDocument * doc = dd->control->document();
    # QTextDocumentLayout * layout = qobject_cast < QTextDocumentLayout * > (doc->documentLayout());
    #
    # // the layout might need to expand the root frame to
    # // the viewport if NoWrap is set
    # if (layout)
    # layout->setViewport(dd->boundingRect);
    #
    # dd->control->drawContents(painter, r);
    #
    # if (layout)
    # layout->setViewport(QRect());
    #
    # painter->restore();
    # }
    #
    # if (option->state & (QStyle::
    #     State_Selected | QStyle::State_HasFocus))
    # qt_graphicsItem_highlightSelected(this, painter, option);
    # }
    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):

        if event.key() == Qt.Key.Key_Escape:
            self.clearFocus()
            # bu asagidaki if var cunku text item bos ise
            # birden fazla defa selectionqueue da remove etmeye calisabiliyor
            # veya item sahneden silindiginden item.scene() none donuyor gibi sebeplerden dolayi.
            # self.childItems() kontrolu de, yazi bir baska nesnenin parenti iken silinmis olabilir diye.
            if self.toPlainText() or self.childItems():
                self.setSelected(False)

        if event.modifiers() == Qt.KeyboardModifier.ControlModifier:
            if event.key() == Qt.Key.Key_Enter or event.key() == Qt.Key.Key_Return:
                self.clearFocus()
                if self.toPlainText() or self.childItems():
                    self.setSelected(False)

        if event.modifiers() == Qt.KeyboardModifier.ShiftModifier:
            if event.key() == Qt.Key.Key_Enter or event.key() == Qt.Key.Key_Return:
                self.clearFocus()
                if self.toPlainText() or self.childItems():
                    self.setSelected(False)
                    self.scene().create_empty_text_object_with_double_click(
                        QPointF(self.sceneLeft(), self.sceneBottom() + 10))

        super(Text, self).keyPressEvent(event)

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):
        rect = self.sceneBoundingRect()

        x = self.scenePos().x()
        y = self.scenePos().y()
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = x - xs
        y = y - ys

        w = rect.width()
        h = rect.height()

        bicimSozluk = self.ver_karakter_bicimi()
        bold = "font-weight:bold;" if bicimSozluk["b"] else ""
        italic = "font-style:italic;" if bicimSozluk["i"] else ""
        underline = "underline" if bicimSozluk["u"] else ""
        strikeOut = "line-through" if bicimSozluk["s"] else ""
        overline = "overline" if bicimSozluk["o"] else ""
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f"text-decoration: {underline} {strikeOut} {overline};"
        else:
            bicimler2 = ""

        renk_arkaPlan = f"({self.arkaPlanRengi.red()},{self.arkaPlanRengi.green()},{self.arkaPlanRengi.blue()},{self.arkaPlanRengi.alpha() / 255})"
        renk_yazi = f"({self.yaziRengi.red()},{self.yaziRengi.green()},{self.yaziRengi.blue()},{self.yaziRengi.alpha() / 255})"

        if self.isPlainText:
            yazi_str = f'<p style=" display: inline-block; ' \
                       f'color:rgba{renk_yazi}; ' \
                       f'margin:0; padding:7px;">{self.text()}</p>'
        else:
            html = self.toHtml()
            body_baslangic = html.find("<bo")
            bas = html.find('">', body_baslangic)
            son = html.find("</bo")
            # yazi = self.toHtml()
            yazi_str = html[bas + 2:son]

        hiza = self.ver_yazi_hizasi()
        # if hiza == Qt.AlignLeft or hiza == Qt.AlignLeft | Qt.AlignVCenter:
        #     yazi_hiza = "left"
        if hiza == Qt.AlignmentFlag.AlignCenter or hiza == Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "center"
        elif hiza == Qt.AlignmentFlag.AlignRight or hiza == Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "right"
        elif hiza == Qt.AlignmentFlag.AlignJustify or hiza == Qt.AlignmentFlag.AlignJustify | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "justify"
        else:
            yazi_hiza = "left"

        div_str = f"""
                    <article style=" 
                     color:rgba{renk_yazi};
                     background:rgba{renk_arkaPlan};
                     font-size:{self.fontPointSizeF()}pt; 
                     font-family:{self.font().family()};
                     text-align: {yazi_hiza};
                     {bicimler1}
                     {bicimler2}
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     top:{y}px;
                     left:{x}px;
                     width:{w};
                     height:{h};
                     transform-box: fill-box;
                     transform-origin: top left;
                     transform: rotate({self.rotation()}deg);

                     " id="{self._kim}">{yazi_str}</article>\n
            """

        return div_str

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget):

        painter.save()
        # painter.fillRect(option.rect, self.arkaPlanRengi)
        painter.fillRect(option.rect, self.brush())

        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
            else:
                selectionPenBottom = self.selectionPenBottom
            painter.setPen(selectionPenBottom)
            painter.drawRect(option.rect)

            ########################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            ########################################################################
            # if not self.isPinned and self.isActiveItem:
            #     # painter.setPen(self.handlePen)
            #     painter.drawRect(self.topLeftHandle)
            #     painter.drawRect(self.topRightHandle)
            #     painter.drawRect(self.bottomRightHandle)
            #     painter.drawRect(self.bottomLeftHandle)
            ########################################################################

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     # painter.setPen(QPen(Qt.GlobalColor.red))
        #     painter.drawRect(option.rect)

        if self.isTextOverflowed:
            painter.setPen(QPen(Qt.GlobalColor.red, 2, Qt.PenStyle.DashLine))
            painter.drawLine(self._rect.bottomLeft(), self._rect.bottomRight())

        # if not self.isPlainText:
        #     # painter.drawEllipse(self.zenginYaziBelirteci)
        #     # painter.drawText(5, 5, "📌🎨☼☀®")
        #     painter.drawText(0, 10, "*")

        # painter.setPen(Qt.blue)
        # painter.drawRect(self.boundingRect())
        # painter.setPen(Qt.green)
        # painter.drawRect(self._rect)
        painter.restore()

        # option2 = QStyleOptionGraphicsItem(option)
        # option2.state = 0
        # option.state &= ~(QStyle.State_Selected | QStyle.State_HasFocus)
        # option2.exposedRect.setSize(self.document().pageSize())
        # option2.exposedRect.setSize(self.document().documentLayout().documentSize())

        # option.exposedRect = self.boundingRect()
        # option.rect = option.exposedRect.toRect()

        # rect = QRectF(self._rect)
        # rect.moveTo(0, 0)
        # option2.exposedRect = rect

        # TODO bunlar niye koyuldu?
        option.exposedRect = self._rect
        option.rect = self._rect.toRect()

        # painter.setClipRect(option.exposedRect)
        super(Text, self).paint(painter, option, widget)

        # self.doc.drawContents(painter, self._rect)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect,
        #                  "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # painter.setPen(QPen(Qt.green, 12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.boundingRect().center())
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.sceneBoundingRect())
        # # # # # # debug end - pos() # # # # #



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/29/22'

from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QLabel, QHBoxLayout, QPushButton, QDockWidget, QWidget


#######################################################################
class DockWidget(QDockWidget):

    # ---------------------------------------------------------------------
    def __init__(self, baslik, parent=None):
        super(DockWidget, self).__init__(parent)

        self.renkYazi = QColor(255, 255, 255)
        self.renkArkaplan = QColor(153, 170, 187)

        self.setContentsMargins(0,0,0,0)

        self.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetMovable |
                         QDockWidget.DockWidgetFeature.DockWidgetClosable |
                         # QDockWidget.DockWidgetFeature.DockWidgetVerticalTitleBar|
                         QDockWidget.DockWidgetFeature.DockWidgetFloatable
                         )

        self.setWindowTitle(baslik)

        # baslik
        baslikWidget = QWidget(self)
        baslikWidget.setFixedHeight(20)
        baslikWidget.setContentsMargins(0, 0, 0, 0)
        baslikWidget.setAutoFillBackground(True)

        p = baslikWidget.palette()
        p.setColor(baslikWidget.foregroundRole(), self.renkYazi)
        p.setColor(baslikWidget.backgroundRole(), self.renkArkaplan)
        baslikWidget.setPalette(p)

        # baslikWidget.setStyleSheet("QWidget {"
        #                            # "font-size:1.3em; "
        #                            # "font-weight:bold;"
        #                            # "margin-right:10px;"
        #                            # "margin-left:10px;"
        #                            # "margin-top:1px;"
        #                            # "padding-top:1px;"
        #                            # "padding-bottom:1px;"
        #                            "color:#fff; "
        #                            # "background-color:#9ab;"
        #                            "}")

        etiket = QLabel(baslik, baslikWidget)
        etiket.setAlignment(Qt.AlignmentFlag.AlignCenter)

        lay = QHBoxLayout(baslikWidget)
        lay.setContentsMargins(0, 0, 0, 0)

        btnKapat = QPushButton("x", baslikWidget)
        btnKapat.setFixedWidth(18)
        btnKapat.setFlat(True)
        btnKapat.setAutoFillBackground(True)
        btnKapat.clicked.connect(self.close)

        p = btnKapat.palette()
        p.setColor(btnKapat.foregroundRole(), self.renkYazi)
        p.setColor(btnKapat.backgroundRole(), self.renkArkaplan)
        btnKapat.setPalette(p)

        btnYuzdur = QPushButton("o", baslikWidget)
        btnYuzdur.setFixedWidth(18)
        btnYuzdur.setFlat(True)
        btnYuzdur.setAutoFillBackground(True)
        btnYuzdur.clicked.connect(lambda: self.setFloating(not self.isFloating()))

        p = btnYuzdur.palette()
        p.setColor(btnYuzdur.foregroundRole(), self.renkYazi)
        p.setColor(btnYuzdur.backgroundRole(), self.renkArkaplan)
        btnYuzdur.setPalette(p)

        # lay.addStretch()
        lay.addSpacing(36)
        lay.addWidget(etiket)
        lay.addWidget(btnYuzdur)
        lay.addWidget(btnKapat)

        baslikWidget.setLayout(lay)

        self.setTitleBarWidget(baslikWidget)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '27/Aug/2016'

from PySide6.QtWidgets import QListWidgetItem, QStyledItemDelegate
from PySide6.QtGui import QPen, QColor
from PySide6.QtCore import Qt


#######################################################################
class ListWidgetItem(QListWidgetItem):
    # ---------------------------------------------------------------------
    def __init__(self, view=None, tip=None):
        super(ListWidgetItem, self).__init__(view, tip)

        self._pen = QPen()
        # self._fontPointSize = 10
        self._presetFont = self.font()
        self._cizgiRengi = QColor(Qt.GlobalColor.transparent)

    # ---------------------------------------------------------------------
    def setCizgiRengi(self, col):
        self._cizgiRengi = col

    # ---------------------------------------------------------------------
    def cizgiRengi(self):
        return self._cizgiRengi

    # ---------------------------------------------------------------------
    def setPresetFont(self, font):
        self._presetFont = font

    # ---------------------------------------------------------------------
    def presetFont(self):
        return self._presetFont

    # # ---------------------------------------------------------------------
    # def setFont(self, font):
    #     # self.setFontPointSize(font.pointSize())
    #     # listede fazla yer kaplamasin diye,
    #     # bu sınıfta kullandıgımız self._fontPointSize, stili uygularken kullanacagimiz kaydedilmis font boyutu
    #     # hemen asagida set settigimiz font boyutu stilin listWidgetItem olarak listwidgette cizdirilen boyutu.
    #     # if self._fontPointSize >= 72:
    #     #     font.setPointSize(72)
    #     font.setPointSize(10)
    #     super(ListWidgetItem, self).setFont(font)

    # ---------------------------------------------------------------------
    def setPen(self, pen):
        if self._pen == pen:
            return
        self._pen = pen

    # ---------------------------------------------------------------------
    def pen(self):
        return self._pen


########################################################################
class ListWidgetItemDelegate(QStyledItemDelegate):
    def __init__(self, parent=None):
        # QStyledItemDelegate.__init__(self)
        super(ListWidgetItemDelegate, self).__init__(parent)
        # self.offset = 2

        # self.linePen = QPen(QColor(Qt.transparent))
        # self.linePen.setWidth(2)

        # self.textPen = QPen(QColor(Qt.black))

        # self.alignment = Qt.AlignHCenter | Qt.AlignVCenter

    def paint(self, painter, option, index):
        super(ListWidgetItemDelegate, self).paint(painter, option, index)

        painter.save()
        # bu alttaki yerine userrole falan mi kullansak
        cizgiRengi = self.parent().item(index.row()).cizgiRengi()
        # linePen = QPen(cizgiRengi)
        # linePen.setWidth(2)
        # linePen.setStyle()
        # linePen.setCapStyle()
        # linePen.setMiterLimit()Style()
        # painter.setPen(linepen)
        painter.setPen(QPen(cizgiRengi, 5))
        # itemRect = option.rect.adjusted(self.offset, self.offset, -self.offset, -self.offset)
        # painter.drawRect(option.rect)
        painter.drawLine(option.rect.topLeft(), option.rect.bottomLeft())
        painter.drawLine(option.rect.topRight(), option.rect.bottomRight())

        painter.restore()
        # text = index.data(Qt.DisplayRole)
        # painter.setPen(self.textpen)
        # painter.drawText(option.rect, self.AlignmentFlag, text)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtWidgets import QSpinBox, QDoubleSpinBox
from PySide6.QtCore import Signal, Slot


#######################################################################
class DoubleSpinBox(QDoubleSpinBox):
    valueChangedFromDoubleSpinBoxGuiNotBySetValue = Signal(float)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(DoubleSpinBox, self).__init__(parent)
        self.kodIleSetEdiliyor = False
        self.valueChanged.connect(self.on_value_changed)

    # ---------------------------------------------------------------------
    def setValue(self, val):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        QDoubleSpinBox.setValue(self, val)
        self.kodIleSetEdiliyor = False

    @Slot(float)
    def on_value_changed(self, val):
        if not self.kodIleSetEdiliyor:
            self.valueChangedFromDoubleSpinBoxGuiNotBySetValue.emit(val)
        # else:
        #     self.valueChangedWithSetValue.emit(val)


#######################################################################
class SpinBox(QSpinBox):
    valueChangedFromSpinBoxGuiNotBySetValue = Signal(int)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(SpinBox, self).__init__(parent)
        self.kodIleSetEdiliyor = False
        self.valueChanged.connect(self.on_value_changed)

    # ---------------------------------------------------------------------
    def setValue(self, val):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        QSpinBox.setValue(self, val)
        self.kodIleSetEdiliyor = False

    @Slot(int)
    def on_value_changed(self, val):
        if not self.kodIleSetEdiliyor:
            self.valueChangedFromSpinBoxGuiNotBySetValue.emit(val)
        # else:
        #     self.valueChangedWithSetValue.emit(val)


#######################################################################
class SpinBoxForRotation(QSpinBox):
    valueChangedFromSpinBoxGuiNotBySetValue = Signal(int)

    # valueChangedWithSetValue = Signal(int)

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(SpinBoxForRotation, self).__init__(parent)
        self.kodIleSetEdiliyor = False
        self.valueChanged.connect(self.on_value_changed)

    # ---------------------------------------------------------------------
    def setValue(self, val):
        self.kodIleSetEdiliyor = True
        # self.setValue(val)
        val %= 360
        if val < 0:
            val += 360
        QSpinBox.setValue(self, val)
        self.kodIleSetEdiliyor = False

    @Slot(int)
    def on_value_changed(self, val):
        val %= 360
        if val < 0:
            val += 360
        QSpinBox.setValue(self, val)
        if not self.kodIleSetEdiliyor:
            self.valueChangedFromSpinBoxGuiNotBySetValue.emit(val)
        # else:
        #     self.valueChangedWithSetValue.emit(val)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'defter'
__date__ = '9/18/23'
__author__ = 'E. Y.'

from PySide6.QtCore import Signal, QSize, Qt, Slot
from PySide6.QtWidgets import QLineEdit, QToolButton, QStyle
from . import icons_rc


#######################################################################
class AraLineEdit(QLineEdit):
    temizleBtnClicked = Signal()
    shiftEnterVeyaReturnBasildi = Signal()
    returnVeyaEnterBasildi = Signal()
    aramaMetniDegisti = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(AraLineEdit, self).__init__(parent)

        self.temizleBtn = QToolButton(self)
        self.temizleBtn.setIconSize(QSize(16, 16))
        # self.temizleBtn = QtGui.QPushButton(self)
        # self.temizleBtn.setText("clear")
        # self.temizleBtn.setIcon(QIcon(':icons/filtre-temizle-aktif.png'))
        self.temizleBtn.setStyleSheet(
            'QToolButton {border: 0px; padding: 0px; image: url(:canta/icons/filtre-temizle-pasif.png);}'
            'QToolButton:hover {image: url(:icons/filtre-temizle-uzerinde.png);}'
            'QToolButton:pressed {image: url(:icons/filtre-temizle-basili.png);}')
        # self.temizleBtn.setCursor(Qt.CursorShape.ArrowCursor)
        self.temizleBtn.clicked.connect(self.act_temizle)

        frameWidth = self.style().pixelMetric(QStyle.PixelMetric.PM_DefaultFrameWidth)
        buttonSize = self.temizleBtn.sizeHint()

        self.setStyleSheet('QLineEdit {padding-right: %dpx; }' % (buttonSize.width() + frameWidth + 1))
        self.setMinimumSize(max(self.minimumSizeHint().width(), buttonSize.width() + frameWidth * 2 + 2),
                            max(self.minimumSizeHint().height(), buttonSize.height() + frameWidth * 2 + 2))

        self.textEdited.connect(self.temizleBtn_durum_guncelle)
        self.textEdited.connect(self.aramaMetniDegisti)

    # ---------------------------------------------------------------------
    def resizeEvent(self, event):
        buttonSize = self.temizleBtn.sizeHint()
        frameWidth = self.style().pixelMetric(QStyle.PixelMetric.PM_DefaultFrameWidth)
        self.temizleBtn.move(self.rect().right() - frameWidth - buttonSize.width(),
                             (self.rect().bottom() - buttonSize.height() + 1) / 2)
        super(AraLineEdit, self).resizeEvent(event)

    # ---------------------------------------------------------------------
    def act_temizle(self):
        self.clear()
        self.temizleBtn_durum_guncelle()
        self.aramaMetniDegisti.emit()

    # ---------------------------------------------------------------------
    @Slot()
    def temizleBtn_durum_guncelle(self):

        if self.text():
            self.temizleBtn.setStyleSheet(
                'QToolButton {border: 0px; padding: 0px; image: url(:icons/filtre-temizle-aktif.png);}'
                'QToolButton:hover {image: url(:icons/filtre-temizle-uzerinde.png);}'
                'QToolButton:pressed {image: url(:icons/filtre-temizle-basili.png);}')

        else:
            self.temizleBtn.setStyleSheet(
                'QToolButton {border: 0px; padding: 0px; image: url(:icons/filtre-temizle-pasif.png);}'
                'QToolButton:hover {image: url(:icons/filtre-temizle-uzerinde.png);}'
                'QToolButton:pressed {image: url(:icons/filtre-temizle-basili.png);}')

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event):
        if event.key() == Qt.Key.Key_Return or event.key() == Qt.Key.Key_Enter:

            if event.modifiers() & Qt.KeyboardModifier.ShiftModifier:
                self.shiftEnterVeyaReturnBasildi.emit()
            else:  # diger modiferlar da kaliyor ama olsun, cok onemli degil..
                self.returnVeyaEnterBasildi.emit()

        super(AraLineEdit, self).keyPressEvent(event)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '04/Sep/2016'

import os

from PySide6.QtCore import Qt, QUrl, QSizeF, QRectF, QPointF, QPoint, QLine
from PySide6.QtGui import QPen, QPixmap, QPixmapCache, QPainterPath, QColor
from PySide6.QtWidgets import QStyle
from PySide6.QtMultimedia import QMediaPlayer, QAudioOutput
from PySide6.QtMultimediaWidgets import QGraphicsVideoItem

from .. import shared
from ..nesneler.base import BaseItem


########################################################################
class GraphicsVideoItem(QGraphicsVideoItem):
    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(GraphicsVideoItem, self).__init__(parent)
        self.ustGrup = None

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, video_kopyalaniyor_mu):
        return ""


########################################################################
class VideoItem(BaseItem):
    # @Slot() kullanmiyouz cunku qobjectten inherit etmiyor.
    # !! videoItem qgraphicsobjecten ediyor, yalniz bu baseItem ikisinden de etmiyor...

    Type = shared.VIDEO_ITEM_TYPE

    # ---------------------------------------------------------------------
    def __init__(self, filePath, pos, rect, yaziRengi, arkaPlanRengi, pen, font, isEmbeded=False, parent=None):
        # if rect.size() == QSizeF(1, 1):
        #     rect
        self.videoItem = GraphicsVideoItem()
        self.videoItem.setFlag(QGraphicsVideoItem.GraphicsItemFlag.ItemStacksBehindParent)
        rect.setSize(self.videoItem.size())

        #
        arkaPlanRengi = QColor(0, 0, 0, 0)
        super(VideoItem, self).__init__(pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent)
        # QGraphicsVideoItem.__init__(parent)
        # super(VideoItem, self).__init__(parent)
        # BaseItem.__init__(pos, rect, arkaPlanRengi, pen, font, parent)

        # self.videoItem = QGraphicsVideoItem(self)
        # self.videoItem = GraphicsVideoItem(self)
        # self.videoItem.setFlag(QGraphicsVideoItem.ItemStacksBehindParent)

        self.kenarPen = QPen(pen.color(), 3, Qt.PenStyle.SolidLine)

        self.videoItem.nativeSizeChanged.connect(self.act_native_size_changed)
        # resolution = media.canonicalResource().resolution()
        # self.videoItem.nativeSizeChanged.connect(self._rect.setSize)
        self.videoItem.setAspectRatioMode(Qt.AspectRatioMode.KeepAspectRatio)

        self.video_hazir_mi = False

        self.videoItem.setParentItem(self)
        # self.player parenti defter.py ekle_video_direkt icinde (scene.py cagiriyor) ekliyoruz,
        # yoksa tab silinince self.player kaliyor,
        # ve video oyamakta iken tab kapanirsa self.player.playbackStateChanged sinyali vermeye devam ediyor
        self.player = QMediaPlayer()
        self.player.setVideoOutput(self.videoItem)

        self.audioOutput = QAudioOutput()
        self.player.setAudioOutput(self.audioOutput)
        self.player.playbackStateChanged.connect(self.act_playback_state_changed)

        self.isEmbeded = isEmbeded
        self.filePathForSave = filePath
        if not isEmbeded:
            self.originalSourceFilePath = filePath

        # url = QUrl("http://")
        if os.path.exists(filePath):
            self.eskiYazi = self.text()
            self.setText("Loading...")
            url = QUrl.fromLocalFile(filePath)
            # media = QMediaContent(url)
            # print(media.canonicalResource().audioCodec())
            # print(media.canonicalResource().videoCodec())
            # print(media.canonicalResource().audioBitRate())
            # print(media.canonicalResource().videoBitRate())
            # print(media.canonicalResource().channelCount())

            # self.player.setMedia(media)
            self.video_poziyon_sure_str = self.video_toplam_sure_str = "00:00:00"
            self.player.hasVideoChanged.connect(self.act_video_changed)
            self.player.positionChanged.connect(self.act_player_position_changed)
            self.player.durationChanged.connect(self.act_player_duration_changed)
            self.player.mediaStatusChanged.connect(self.act_player_media_status_changed)
            self.player.errorOccurred.connect(self.act_player_error_occured)
            self.player.errorChanged.connect(self.act_player_error_changed)
            self.player.seekableChanged.connect(self.act_seekable_changed)
            self.player.setSource(url)

            self.playControlsY = self._rect.bottom() - 50
            # self.playControlsY = self.videoItem.size().height() / 2
            self.ikonBoyutu = 20
            self.yariIkonBoyutu = self.ikonBoyutu / 2
            self.boundingRectTasmaDegeri = max(self.handleSize, self._pen.widthF() / 2, self.yariIkonBoyutu)
            # print(self.playControlsY)

            self.sliderLine = QLine(self.videoItem.pos().x() + 0,
                                    self.playControlsY,
                                    self.videoItem.size().width(),
                                    self.playControlsY)

            self.videoSliderEllipseRect = QRectF(0, self.playControlsY,
                                                 self.ikonBoyutu,
                                                 self.ikonBoyutu)
            self.videoIkonMerkezNoktasi = QPointF(0, self.playControlsY)

            self.daire_ici_ikonlari_olustur()

            self.videoSliderEllipseFirstClickPos = QPointF()
            self.videoSliderEllipseStartDrag = False
            self.audioSliderEllipseRect = QRectF(self._rect.center().x(),
                                                 # (100 / self._rect.height() / self.audioOutput.volume()),
                                                 self._rect.height() - self._rect.height() * self.audioOutput.volume(),
                                                 self.ikonBoyutu,
                                                 self.ikonBoyutu)

            self.audioSliderEllipseStartDrag = False

            self.isVideoSliderHovered = False
            self.isAudioSliderHovered = False

        else:
            self.filePathForDraw = ':icons/warning.png'
            self.pixmap = QPixmap()
            # print(QPixmapCache.cacheLimit())
            # self.pixmap = QPixmapCache.find(self.filePathForDraw, self.pixmap)
            self.pixmap = QPixmapCache.find(self.filePathForDraw)
            if not self.pixmap:
                self.pixmap = QPixmap(self.filePathForDraw)
                QPixmapCache.insert(self.filePathForDraw, self.pixmap)

            # !!!! paint method override
            self.paint = self.paint_warning_image
            self.mousePressEvent = super(VideoItem, self).mousePressEvent
            self.mouseMoveEvent = super(VideoItem, self).mouseMoveEvent
            self.mouseReleaseEvent = super(VideoItem, self).mouseReleaseEvent
            self.hoverMoveEvent = super(VideoItem, self).hoverMoveEvent

    # ---------------------------------------------------------------------
    def kareAl(self):

        # self.player.setPosition()
        image = self.player.videoSink().videoFrame().toImage()
        return image

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        if self._rect == rect:
            return
        self.prepareGeometryChange()
        self._rect = rect
        self._boundingRect = QRectF()
        self.update()
        # self.videoItem.setSize(rect.size())

    # ---------------------------------------------------------------------
    def keyPressEvent(self, event) -> None:
        if event.key() == Qt.Key.Key_Right:
            self.player.setPosition(self.player.position() + 10000)
        elif event.key() == Qt.Key.Key_Left:
            self.player.setPosition(self.player.position() - 10000)
        elif event.key() == Qt.Key.Key_Space:
            if self.player.playbackState() == QMediaPlayer.PlaybackState.StoppedState \
                    or self.player.playbackState() == QMediaPlayer.PlaybackState.PausedState:
                self.player.play()
            else:
                # self.player.stop()
                self.player.pause()

        event.accept()

    # super(VideoItem, self).keyPressEvent(event)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base- text -path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_video_sag_menu_about_to_show(self)
        self.scene().parent().videoSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def act_player_media_status_changed(self, status):
        # print(status)
        pass

    # ---------------------------------------------------------------------
    def act_player_error_occured(self, error, errorString):
        # print(error)
        # print(errorString)
        pass

    # ---------------------------------------------------------------------
    def act_player_error_changed(self):
        # print("degisti error")
        pass

    # ---------------------------------------------------------------------
    def act_seekable_changed(self, seekable):
        # print(seekable, "seekable")
        pass

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return VideoItem.Type

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "isEmbeded": self.isEmbeded,
                      "filePath": self.filePathForSave,
                      "originalSourceFilePath": self.originalSourceFilePath,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "pen": self._pen,
                      "font": self._font,
                      # "imageOpacity": self.imageOpacity,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        return properties

    # ---------------------------------------------------------------------
    def varsaEnUsttekiGrubuGetir(self):
        parentItem = self.parentItem()
        while parentItem:
            if parentItem.type() == shared.GROUP_ITEM_TYPE:
                return parentItem
            parentItem = parentItem.parentItem()
        return None

    # ---------------------------------------------------------------------
    def nesne_sahneden_silinmek_uzere(self):
        # playerPositionChanged sinyali gitmeye devam ediyor ve nesne "already deleted" hatası.
        self.player.stop()

    # ---------------------------------------------------------------------
    def boundingRect(self):
        # if self._boundingRect.isNull():
        self._boundingRect = QRectF(self.rect())
        return self._boundingRect.adjusted(-self.boundingRectTasmaDegeri, -self.boundingRectTasmaDegeri,
                                           self.boundingRectTasmaDegeri, self.boundingRectTasmaDegeri)

    # ---------------------------------------------------------------------
    def setCizgiKalinligi(self, width):
        self._pen.setWidthF(width)
        self.textPen.setWidthF(width)
        self.boundingRectTasmaDegeri = max(self.handleSize, self._pen.widthF() / 2, self.yariIkonBoyutu)
        self.update()

    # ---------------------------------------------------------------------
    def daire_ici_ikonlari_olustur(self):

        olcekli_mesafe = self.ikonBoyutu / 4

        self.videoPlayIkonu = QPainterPath()
        self.videoPlayIkonu.moveTo(olcekli_mesafe, olcekli_mesafe)
        self.videoPlayIkonu.lineTo(olcekli_mesafe, self.ikonBoyutu - olcekli_mesafe)
        self.videoPlayIkonu.lineTo(self.ikonBoyutu - olcekli_mesafe, self.ikonBoyutu / 2)
        self.videoPlayIkonu.lineTo(olcekli_mesafe, olcekli_mesafe)

        self.videoPauseIkonu = QPainterPath()
        self.videoPauseIkonu.moveTo(olcekli_mesafe, olcekli_mesafe)
        self.videoPauseIkonu.lineTo(olcekli_mesafe, self.ikonBoyutu - olcekli_mesafe)
        self.videoPauseIkonu.moveTo(3 * olcekli_mesafe, olcekli_mesafe)
        self.videoPauseIkonu.lineTo(3 * olcekli_mesafe, self.ikonBoyutu - olcekli_mesafe)

        self.videoStopIkonu = QPainterPath()
        self.videoStopIkonu.addRect(olcekli_mesafe,
                                    olcekli_mesafe,
                                    self.ikonBoyutu - 2 * olcekli_mesafe,
                                    self.ikonBoyutu - 2 * olcekli_mesafe)

        # self.videoBosDaireIkonu = QPainterPath()
        # self.videoBosDaireIkonu.addEllipse(self.videoSliderEllipseRect)

        self.videoIkonu = self.videoStopIkonu

    # ---------------------------------------------------------------------
    def act_playback_state_changed(self, pbstate):

        if pbstate == QMediaPlayer.PlaybackState.StoppedState:
            self.videoIkonu = self.videoStopIkonu
        elif pbstate == QMediaPlayer.PlaybackState.PlayingState:
            self.videoIkonu = self.videoPlayIkonu
        elif pbstate == QMediaPlayer.PlaybackState.PausedState:
            self.videoIkonu = self.videoPauseIkonu
        # else:
        #     # buraya gelmesi beklenmemekte, ilerki qt surumlerinde ek durumlar gelirse..
        #     self.videoIkonu = self.videoBosDaireIkonu

    # ---------------------------------------------------------------------
    def act_video_changed(self, video_hazir_mi):
        self.setText(self.eskiYazi)
        self.video_hazir_mi = video_hazir_mi
        # self.player.play()

    # ---------------------------------------------------------------------
    def act_native_size_changed(self, nativeSize):
        if not nativeSize.isEmpty():  # videoItem silinice bos QSizeF() gonderiyor.
            self._rect.setSize(nativeSize.scaled(self.videoItem.size(), Qt.AspectRatioMode.KeepAspectRatio))
            self.update_elements_after_resize()

    # ---------------------------------------------------------------------
    def update_elements_after_resize(self):
        self.update_resize_handles()
        self.scene().parent().change_transform_box_values(self)
        # self.setPos(x, y)
        # return QGraphicsItem.mouseMoveEvent(self, event)

        self.update_painter_text_rect()

        self.playControlsY = self.videoItem.size().height() / 2

        self.videoItem.setPos(self._rect.topLeft())
        self.videoItem.setSize(self._rect.size())
        # self.sliderLine.setLength()
        # self.playControlsY = self._rect.bottom() - 50
        # self.playControlsY = self.videoItem.boundingRect().center().y()
        # self.playControlsY = 100
        self.sliderLine.setPoints(QPoint(self.videoItem.pos().x(), self.playControlsY),
                                  QPoint(self._rect.right() + self.videoItem.pos().x(), self.playControlsY))

        # TODO: hmm kirmiz da asagilara falan da tasinabilsin ,,
        #  ama tabi x ekseni gecerli olsun mavi de oyle ters eksende.
        # self.videoSliderEllipseRect.moveCenter(QPointF(event.pos().x(), self.playControlsY))
        # TODO: aynı zamanda y oranını tekrar hesaplamak lazım.
        self.audioSliderEllipseRect.moveCenter(QPointF(self._rect.center().x(),
                                                       self._rect.height() - self._rect.height() * self.audioOutput.volume()))

        if self.player.duration():  # olasi bir sifira bolmek durumu olmasin diye
            self.videoSliderEllipseRect.moveCenter(
                QPointF(self.videoItem.pos().x() + (
                        self.videoItem.size().width() * self.player.position() / self.player.duration()),
                        self.playControlsY))

    # ---------------------------------------------------------------------
    def itemChange(self, change, value):
        if change == BaseItem.GraphicsItemChange.ItemSelectedChange:
            if value:
                self.scene().parent().item_selected(self)
            else:
                self.scene().parent().item_deselected(self)
        elif change == BaseItem.GraphicsItemChange.ItemSceneChange:
            # The value argument is the new scene or a null pointer if the item is removed from a scene.
            # When item removed from the scene, we stop the player, to prevent stream errors.
            if not value:
                self.player.stop()
        return value
        # super(Rect, self).itemChange(change, value)

    # ---------------------------------------------------------------------
    # @Slot(int)  # qint64 # hata oluyor su an, bug varmis.
    def act_player_position_changed(self, pos_ms):
        # yuzde = pos / self.player.duration() * 100
        # self.sliderEllipseCenterX = self.videoItem.size().width() / 100 * yuzde
        # self.sliderEllipseCenter = QPointF(self.videoItem.size().width() * pos / self.player.duration(), 100)

        # TODO:
        """
        self.videoSliderEllipseRect.moveCenter(QPointF(self.videoItem.size().width() * pos / self.player.duration(),
         self.playControlsY))
        ZeroDivisionError: float division by zero
        """
        # print(pos)
        # return
        if self.player.duration():
            self.videoSliderEllipseRect.moveCenter(
                QPointF(self.videoItem.pos().x() + (self.videoItem.size().width() * pos_ms / self.player.duration()),
                        self.playControlsY))
            # print(self.playControlsY)

        dk, sn = divmod((pos_ms // 1000), 60)
        saat, dk = divmod(dk, 60)
        self.video_poziyon_sure_str = f"{saat:02}:{dk:02}:{sn:02}"

    # ---------------------------------------------------------------------
    def act_player_duration_changed(self, toplam_sure_ms):
        dk, sn = divmod((toplam_sure_ms // 1000), 60)
        saat, dk = divmod(dk, 60)
        self.video_toplam_sure_str = f"{saat:02}:{dk:02}:{sn:02}"

    # ---------------------------------------------------------------------
    def childItems(self):
        liste = super(VideoItem, self).childItems()
        liste.remove(self.videoItem)
        return liste

    # ---------------------------------------------------------------------
    def mousePressEvent(self, event):
        super(VideoItem, self).mousePressEvent(event)

        if self.videoSliderEllipseRect.contains(event.pos()):
            # print(self.player.playbackState(), 1)
            if self.player.playbackState() == QMediaPlayer.PlaybackState.StoppedState \
                    or self.player.playbackState() == QMediaPlayer.PlaybackState.PausedState:
                self.player.play()
                # print(self.player.playbackState(), 2)
            else:
                # self.player.stop()
                self.player.pause()
            self.videoSliderEllipseStartDrag = True
            self.yeniPlayerPosition = None
            self.videoSliderEllipseFirstClickPos = event.pos()

        if self.audioSliderEllipseRect.contains(event.pos()):
            self.audioSliderEllipseStartDrag = True

    # ---------------------------------------------------------------------
    def mouseMoveEvent(self, event):

        if self.videoSliderEllipseStartDrag:
            if abs(self.videoSliderEllipseFirstClickPos.manhattanLength() - event.pos().manhattanLength()) > 0:
                self.player.pause()
                x = min(max(event.pos().x(), 0), self.videoItem.size().width())
                self.videoSliderEllipseRect.moveCenter(QPointF(x, self.playControlsY))
                # ~ self.videoSliderEllipseRect.moveCenter(event.pos())
                # yuzde = pos / self.player.duration() * 100
                # self.sliderEllipseCenterX = self.videoItem.size().width() / 100 * yuzde

                oran = x / self.videoItem.size().width()
                self.yeniPlayerPosition = int(oran * self.player.duration())
                # print(duration, "/", self.player.duration())

                # self.player.setPosition(int(self.yeniPlayerPosition))
                # self.player.play()

                # print(self.player.position(), self.yeniPlayerPosition)
                dk, sn = divmod((self.yeniPlayerPosition // 1000), 60)
                saat, dk = divmod(dk, 60)
                self.video_poziyon_sure_str = f"{saat:02}:{dk:02}:{sn:02}"
                self.update()

                # self.playControlsY = event.pos().y()

                # self.player.setPosition(event.pos().x()/self.videoItem.size().width() * 100 * self.player.duration())
            return

        if self.audioSliderEllipseStartDrag:
            yuzde = event.pos().y() / self._rect.height()
            self.audioOutput.setVolume(1 - yuzde)
            # if self.yariIkonBoyutu < event.pos().y() < self._rect.size().height() - self.yariIkonBoyutu:
            if 0 < event.pos().y() < self._rect.size().height():
                self.audioSliderEllipseRect.moveCenter(QPointF(self._rect.center().x(), event.pos().y()))

            return

        # burda super(VideoItem, self).mouseMoveEvent(event) cagirmiyoruz
        # shift + resize durumu videoda yok mesela o yuzden.
        if self._resizing:
            # cunku px=1 , py=1 olarak basliyor cizmeye. burda sifirliyoruz eger ilk sahneye ekleme cizimi ise.
            if not self._eskiRectBeforeResize.size() == QSizeF(1, 1):
                px = self._fixedResizePoint.x()
                py = self._fixedResizePoint.y()
            else:
                px = py = 0
            # mx = event.scenePos().x()
            # my = event.scenePos().y()
            mx = event.pos().x()
            my = event.pos().y()
            topLeft = QPointF(min(px, mx), min(py, my))  # top-left corner (x,y)
            bottomRight = QPointF(max(px, mx), max(py, my))  # bottom-right corner (x,y)
            # size = QSizeF(fabs(mx - px), fabs(my - py))
            # rect = QRectF(topLeft, size)

            rect = QRectF(topLeft, bottomRight)

            c = self._rect.center()

            # Alt Key - to resize around center.
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            # ---------------------------------------------------------------------
            tl = self._rect.topLeft()
            tr = self._rect.topRight()
            br = self._rect.bottomRight()
            bl = self._rect.bottomLeft()
            c = self._rect.center()

            yeniSize = rect.size()
            eskiSize = self._rect.size()
            # eskiSize.scale(yeniSize, Qt.KeepAspectRatio)
            eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.AspectRatioMode.KeepAspectRatioByExpanding)
            # eskiSize.scale(yeniSize.height(), yeniSize.height(), Qt.KeepAspectRatio)

            # if not eskiSize.isNull():
            if not eskiSize.isEmpty():
                self.yedekSize = QSizeF(eskiSize)

            else:
                eskiSize = QSizeF(self.yedekSize)

            rect.setSize(eskiSize)
            h = rect.height()
            w = rect.width()

            if self._resizingFrom == 1:
                rect.moveTopLeft(QPointF(br.x() - w, br.y() - h))

            if self._resizingFrom == 2:
                rect.moveTopRight(QPointF(bl.x() + w, bl.y() - h))

            if self._resizingFrom == 3:
                rect.moveBottomRight(QPointF(tl.x() + w, tl.y() + h))

            elif self._resizingFrom == 4:
                rect.moveBottomLeft(QPointF(tr.x() - w, tr.y() + h))

            # Alt Key - to resize around center
            if event.modifiers() == Qt.KeyboardModifier.AltModifier:
                rect.moveCenter(c)

            self.setRect(rect)  # mouse release eventten gonderiyoruz undoya

            self.update_elements_after_resize()

        # event.accept()
        else:
            super(BaseItem, self).mouseMoveEvent(event)

    # ---------------------------------------------------------------------
    def mouseReleaseEvent(self, event):
        super(VideoItem, self).mouseReleaseEvent(event)
        # self.videoItem.setPos(0, 0)
        self.videoItem.setPos(self._rect.topLeft())
        # if self.videoSliderEllipseRect.contains(event.pos()):
        if self.videoSliderEllipseStartDrag:
            if self.yeniPlayerPosition:
                self.player.blockSignals(True)
                self.player.setPosition(int(self.yeniPlayerPosition))
                self.player.blockSignals(False)
                self.update()
                self.player.play()
            self.videoSliderEllipseStartDrag = False

        # if self.audioSliderEllipseRect.contains(event.pos()):
        if self.audioSliderEllipseStartDrag:
            self.audioSliderEllipseStartDrag = False

    # ---------------------------------------------------------------------
    def hoverEnterEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(VideoItem, self).hoverEnterEvent(event)

    # ---------------------------------------------------------------------
    def hoverMoveEvent(self, event):
        # self.sagUstKare.hide()

        # cursor = self.scene().parent().cursor()

        if self.isSelected():
            if self.videoSliderEllipseRect.contains(event.pos()):
                self.isVideoSliderHovered = True
            else:
                self.isVideoSliderHovered = False

            if self.audioSliderEllipseRect.contains(event.pos()):
                self.isAudioSliderHovered = True
            else:
                self.isAudioSliderHovered = False

        super(VideoItem, self).hoverMoveEvent(event)

    # ---------------------------------------------------------------------
    def hoverLeaveEvent(self, event):
        # event propagationdan dolayi bos da olsa burda implement etmek lazim
        # mousePress,release,move,doubleclick de bu mantıkta...
        # baska yerden baslayip mousepress ile , mousemove baska, mouseReleas baska widgetta gibi
        # icinden cikilmaz durumlara sebep olabiliyor.
        # ayrica override edince accept() de edilmis oluyor mouseeventler
        super(VideoItem, self).hoverLeaveEvent(event)

    # ---------------------------------------------------------------------
    def ses_ayarla(self, delta):

        if delta > 0:
            self.audioOutput.setVolume(self.audioOutput.volume() + 0.025)

        else:
            self.audioOutput.setVolume(self.audioOutput.volume() - 0.025)

        self.audioSliderEllipseRect.moveCenter(QPointF(self._rect.center().x(),
                                                       self._rect.size().height() - self._rect.size().height() * self.audioOutput.volume()))

    # ---------------------------------------------------------------------
    def wheelEvent(self, event):
        # factor = 1.41 ** (event.delta() / 240.0)

        if self.ustGrup:
            return BaseItem.wheelEvent(self, event)

        toplam = event.modifiers().value

        # ctrl = int(Qt.ControlModifier)
        shift = Qt.KeyboardModifier.ShiftModifier.value
        alt = Qt.KeyboardModifier.AltModifier.value

        ctrlAlt = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value
        ctrlShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        altShift = Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value
        ctrlAltShift = Qt.KeyboardModifier.ControlModifier.value + Qt.KeyboardModifier.AltModifier.value + Qt.KeyboardModifier.ShiftModifier.value

        # if event.modifiers() & Qt.ControlModifier:
        if toplam == ctrlShift:
            self.scaleItemByResizing(event.delta())

        # elif event.modifiers() & Qt.ShiftModifier:
        elif toplam == shift:
            self.changeFontSizeF(event.delta())

        # elif event.modifiers() & Qt.AltModifier:
        elif toplam == alt:
            self.changeBackgroundColorAlpha(event.delta())

        elif toplam == ctrlAlt:
            self.changeTextColorAlpha(event.delta())

        elif toplam == ctrlAltShift:
            self.rotateItem(event.delta())
        #
        elif toplam == altShift:
            # self.changeImageItemTextBackgroundColorAlpha(event.delta())
            self.changeLineColorAlpha(event.delta())

        # elif toplam == ctrlAltShift:
        #     self.scaleItemByResizing(event.delta())

        else:
            self.ses_ayarla(event.delta())
            # super(VideoItem, self).wheelEvent(event)

    # ---------------------------------------------------------------------
    def paint_warning_image(self, painter, option, widget=None):
        painter.setClipRect(option.exposedRect)
        painter.drawPixmap(self._rect.toRect(), self.pixmap)

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):
        # super(VideoItem, self).paint(painter, option, widget)

        if not self._pen.width():
            painter.setPen(Qt.PenStyle.NoPen)
        else:
            painter.setPen(self._pen)
        painter.setBrush(self._brush)

        painter.setPen(self.kenarPen)
        painter.drawRect(self._rect)

        if self._text:
            # painter.setWorldMatrixEnabled(False)
            painter.save()
            painter.setFont(self._font)
            # we recreate textPen from same exact color. otherwise, color's alpha not working.
            painter.setPen(self.textPen)
            painter.translate(self._rect.center())
            painter.rotate(-self.rotation())

            painter.translate(-self._rect.center())

            painter.drawText(self.painterTextRect, self._text, self.painterTextOption)

            # painter.drawText(10,10, txt)
            painter.restore()
            # painter.setWorldMatrixEnabled(True)

        # if option.state & QStyle.State_MouseOver:
        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            # painter.setPen(QPen(option.palette.windowText(), 0, Qt.DashLine))
            # painter.setPen(QPen(option.palette.highlight(), 0, Qt.DashLine))

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawRect(self.rect())

        if option.state & QStyle.StateFlag.State_MouseOver:
            painter.setOpacity(0.7)
            painter.setPen(QPen(Qt.GlobalColor.red))
            # if self.isSelected():
            # if option.state & QStyle.State_Selected:
            #     painter.setBrush(QBrush(QColor(255, 50, 50)))
            painter.drawLine(self.sliderLine)
            if self.isVideoSliderHovered:
                painter.setOpacity(1)
            painter.drawEllipse(self.videoSliderEllipseRect)
            painter.save()
            painter.translate(self.videoSliderEllipseRect.topLeft())
            painter.drawPath(self.videoIkonu)
            painter.restore()
            painter.drawText(self._rect, Qt.AlignmentFlag.AlignLeft, f"{self.video_poziyon_sure_str} - {self.video_toplam_sure_str}")
            painter.setPen(QPen(Qt.GlobalColor.blue))
            painter.setOpacity(0.7)
            if self.isAudioSliderHovered:
                painter.setOpacity(1)
            painter.drawEllipse(self.audioSliderEllipseRect)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect, "{0:.2f},  {1:.2f} pos \n{2:.2f},  {3:.2f} spos".format(p.x(), p.y(), s.x(), s.y()))
        # # # t = self.transformOriginPoint()
        # # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f} map".format(mapped.x(), mapped.y()))
        # painter.drawEllipse(self.scenePos(), 10, 10)
        # painter.setPen(Qt.blue)
        # painter.drawEllipse(self.mapFromScene(self.pos()), 10, 10)
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # painter.setPen(QPen(Qt.red,17))
        # painter.drawPoint(self._rect.center())
        # painter.setPen(QPen(Qt.green,12))
        # painter.drawPoint(self.mapFromScene(self.sceneBoundingRect().center()))
        # painter.setPen(QPen(Qt.blue,8))
        # painter.drawPoint(self.sceneBoundingRect().center())
        # painter.drawRect(self.sceneBoundingRect())
        # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, video_kopyalaniyor_mu):
        video_adi = os.path.basename(self.filePathForSave)
        video_adres = self.filePathForSave
        if not html_klasor_kayit_adres:  # def dosyasi icine kaydet
            if self.isEmbeded:
                video_adres = os.path.join("videos", video_adi)
        else:  # dosya html olarak baska bir yere kaydediliyor
            # kopyalanmazsa da, zaten embed olmayan video normal hddeki adresten yuklenecektir.
            if video_kopyalaniyor_mu:
                # iptal: if not self.isEmbeded:  # embed ise zaten tmp klasorden hedef klasore baska metodta kopylanıyor hepsi.
                # embed veya degil, yukarda bahsettigi gibi video kopyalansa da adresi de guncellemek lazim.
                video_adres = os.path.join(html_klasor_kayit_adres, "videos", video_adi)

        # video_str = f'<video src="{self.filePathForSave}" width:{self.videoItem.size().width()};' \
        #             f' height{self.videoItem.size().height()}"></video>'
        video_str = f'<video style="width:100%; height:100%;" controls> <source src="{video_adres}"></video>'

        # rect = self.sceneBoundingRect()
        # xr = rect.left()
        # yr = rect.top()
        # xs = self.scene().sceneRect().x()
        # ys = self.scene().sceneRect().y()
        # x = xr - xs
        # y = yr - ys

        w = self._rect.width()
        h = self._rect.height()
        c = self.sceneBoundingRect().center()
        xr = c.x() - w / 2
        yr = c.y() - h / 2
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = xr - xs
        y = yr - ys

        bicimSozluk = self.ver_karakter_bicimi()
        bold = "font-weight:bold;" if bicimSozluk["b"] else ""
        italic = "font-style:italic;" if bicimSozluk["i"] else ""
        underline = "underline" if bicimSozluk["u"] else ""
        strikeOut = "line-through" if bicimSozluk["s"] else ""
        overline = "overline" if bicimSozluk["o"] else ""
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f"text-decoration: {underline} {strikeOut} {overline};"
        else:
            bicimler2 = ""

        renk_arkaPlan = f"({self.arkaPlanRengi.red()},{self.arkaPlanRengi.green()},{self.arkaPlanRengi.blue()},{self.arkaPlanRengi.alpha() / 255})"
        renk_yazi = f"({self.yaziRengi.red()},{self.yaziRengi.green()},{self.yaziRengi.blue()},{self.yaziRengi.alpha() / 255})"

        hiza = self.ver_yazi_hizasi()
        # if hiza == Qt.AlignLeft or hiza == Qt.AlignLeft | Qt.AlignVCenter:
        #     yazi_hiza = "left"
        if hiza == Qt.AlignmentFlag.AlignCenter or hiza == Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "center"
        elif hiza == Qt.AlignmentFlag.AlignRight or hiza == Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "right"
        elif hiza == Qt.AlignmentFlag.AlignJustify or hiza == Qt.AlignmentFlag.AlignJustify | Qt.AlignmentFlag.AlignVCenter:
            yazi_hiza = "justify"
        else:
            yazi_hiza = "left"

        div_str = f"""
                    <div style="
                     background:rgba{renk_arkaPlan};
                     color:rgba{renk_yazi};
                     font-size:{self.fontPointSizeF()}pt; 
                     font-family:{self.font().family()};
                     text-align: {yazi_hiza};
                     {bicimler1}
                     {bicimler2}
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{self._rect.width()}px;
                     height:{self._rect.height()}px;
                     top:{y}px;
                     left:{x}px;
                     transform-box: fill-box;
                     transform-origin: center;
                     transform: rotate({self.rotation()}deg);" id="{self._kim}">{video_str}{self.text()}</div>\n"""

        # /*background-image:url('{resim_adres}');*/
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '06/Nov/2018'
__author__ = 'Erdinç Yılmaz'
__license__ = 'GPLv3'

import os
import platform
import subprocess

from PySide6.QtCore import Qt, QSize, QFileInfo, QBuffer, QIODevice
from PySide6.QtGui import QPixmap, QColor
from PySide6.QtWidgets import QStyle, QFileIconProvider

from .. import shared
from ..nesneler.base import BaseItem


########################################################################
class DosyaNesnesi(BaseItem):
    Type = shared.DOSYA_ITEM_TYPE

    # ---------------------------------------------------------------------
    # def __init__(self, filePath, pixmap, rect, arkaPlanRengi, cizgiRengi, parent=None, scene=None):
    def __init__(self, dosyaAdresi, pos, rect, yaziRengi, arkaPlanRengi, pen, font, pixmap=None, isEmbeded=False,
                 parent=None):
        super(DosyaNesnesi, self).__init__(pos, rect, yaziRengi, arkaPlanRengi, pen, font, parent)

        if os.path.isfile(dosyaAdresi):
            iconProvider = QFileIconProvider()
            self.ikon = iconProvider.icon(QFileInfo(dosyaAdresi))
            self.ikonPixmap = self.ikon.pixmap(self._rect.toRect().size())
        else:
            self.ikonPixmap = QPixmap(':icons/warning.png')

        self.ikonBoyutu = QSize(32, 32)

        self.renk_yazi_alti_kutusu = QColor(255, 255, 255, 155)

        # ikon altinda bir pixmap daha cizmisiz ama niye? iptal edilme adayi...
        if not pixmap:
            self.pixmap = QPixmap()
        else:
            self.pixmap = pixmap
        self.setText(dosyaAdresi)

        self.isEmbeded = isEmbeded
        self.filePathForSave = dosyaAdresi
        if not isEmbeded:
            self.originalSourceFilePath = dosyaAdresi
        else:
            self.originalSourceFilePath = None

    # ---------------------------------------------------------------------
    def type(self):
        # Enable the use of qgraphicsitem_cast with this item.
        return DosyaNesnesi.Type

    # ---------------------------------------------------------------------
    def get_properties_for_save_binary(self):
        properties = {"type": self.type(),
                      "kim": self._kim,
                      "isEmbeded": self.isEmbeded,
                      "filePath": self.filePathForSave,
                      "originalSourceFilePath": self.originalSourceFilePath,
                      "rect": self._rect,
                      "pos": self.pos(),
                      "rotation": self.rotation(),
                      "zValue": self.zValue(),
                      "yaziRengi": self.yaziRengi,
                      "arkaPlanRengi": self.arkaPlanRengi,
                      "pen": self._pen,
                      "font": self._font,
                      # "imageOpacity": self.imageOpacity,
                      "yaziHiza": int(self.painterTextOption.alignment()),
                      "text": self.text(),
                      "isPinned": self.isPinned,
                      "command": self._command,
                      }
        return properties

    # ---------------------------------------------------------------------
    def setRect(self, rect):
        super(DosyaNesnesi, self).setRect(rect)
        if os.path.isfile(self.filePathForSave):
            self.ikonPixmap = self.ikon.pixmap(self._rect.toRect().size())
        else:
            self.ikonPixmap = QPixmap(':icons/warning.png')

    # ---------------------------------------------------------------------
    def mouseDoubleClickEvent(self, event):
        sistem = platform.system()
        if sistem == "Darwin":
            subprocess.call(('open', self.filePathForSave))
        elif sistem == 'Windows':
            try:
                os.startfile(self.filePathForSave)
            except AttributeError:
                subprocess.call(['open', self.filePathForSave])

        elif sistem == "Linux":
            # subprocess.call(('xdg-open', norm))
            try:
                subprocess.Popen(('xdg-open', self.filePathForSave))
            except Exception as e:
                if subprocess.call(["which", "xdg-open"]) == 1:
                    self.log('---------------------------------------\nPlease install "xdg-open" from your '
                             'package manager. \nMore info at :\nhttps://wiki.archlinux.org/index.php/Xdg-open'
                             '\nor\nhttp://portland.freedesktop.org/xdg-utils-1.0/xdg-open.html\n'
                             '---------------------------------------', 0)
        self.scene().parent().log("File is opening in external editor.", 5000, toStatusBarOnly=True)

        super(BaseItem, self).mouseDoubleClickEvent(event)

    # ---------------------------------------------------------------------
    def contextMenuEvent(self, event):
        # TODO: bu alttaki iki satir aktif idi, ve de ctrl harici sag tiklayinca
        # (base - text - path -group nesnelerinin contextMenuEventinde var)
        # mesela birden fazla nesne secili ve de gruplayacagız diyelim sag menu ile
        # ctrl basılı degil ise tikladigimiz secili kaliyor digerleri siliniyordu
        # uygunsuz bir kullanıcı deneyimi, niye yaptık acaba boyleyi hatırlayana kadar kalsın burda :)
        # if not event.modifiers() == Qt.KeyboardModifier.ControlModifier:
        #     self.scene().clearSelection()
        self.setSelected(True)

        # self.scene().parent().item_context_menu(self, event.screenPos())
        self.scene().parent().on_dosya_sag_menu_about_to_show(self)
        self.scene().parent().dosyaSagMenu.popup(event.screenPos())

        event.accept()

    # ---------------------------------------------------------------------
    def paint(self, painter, option, widget=None):

        # painter.setClipRect(option.exposedRect)
        # painter.setClipRegion(QRegion(option.exposedRect.toRect()))

        painter.setFont(self._font)
        if not self._pen.width():
            painter.setPen(Qt.PenStyle.NoPen)
        else:
            painter.setPen(self._pen)
        # painter.setPen(QPen(self.cizgiRengi))
        painter.setBrush(self._brush)
        # !! eger kareyi burda cizersek, az sonra cizilen textin alphasinda problem oluyor.
        #   alphayi azalttikca yazi siyahlasiyor, en sonda 0 olunca kayboluyor.
        #   -- o yuzden texti cidikten sonra ciziyoruz bu kareyi deyip bunu iptal etmistik--
        #   ama self.cizgiRengini, QColor().fromrgb ile teker teker aynen yeniden olsuturunca
        #   ve bu yeni kopya renk ile pen olusturunca calısıyor. self.setCizgRengi nde bu islem,
        painter.drawRect(self._rect)
        painter.drawPixmap(self._rect.toRect(), self.pixmap)
        painter.drawPixmap(self._rect.toRect(), self.ikonPixmap)
        # painter.drawPixmap(0,0,self.ikonBoyutu.width(), self.ikonBoyutu.height(), self.ikon.pixmap(self.ikonBoyutu))

        # painter.setWorldMatrixEnabled(False)

        painter.setBrush(self.renk_yazi_alti_kutusu)
        painter.setPen(Qt.PenStyle.NoPen)
        painter.drawRect(self._rect.toRect())

        painter.save()
        # we recreate textPen from same exact color. otherwise, color's alpha not working.
        painter.setPen(self.textPen)
        painter.translate(self._rect.center())
        painter.rotate(-self.rotation())
        painter.translate(-self._rect.center())
        painter.drawText(self.painterTextRect, self.filePathForSave, self.painterTextOption)
        painter.restore()
        # painter.setWorldMatrixEnabled(True)

        # if option.state & QStyle.State_MouseOver:
        if option.state & QStyle.StateFlag.State_Selected or self.cosmeticSelect:
            # painter.setPen(QPen(option.palette.windowText(), 0, Qt.DashLine))
            # painter.setPen(QPen(option.palette.highlight(), 0, Qt.DashLine))

            if self.isActiveItem:
                selectionPenBottom = self.selectionPenBottomIfAlsoActiveItem
            else:
                selectionPenBottom = self.selectionPenBottom

            painter.setBrush(Qt.BrushStyle.NoBrush)

            painter.setPen(selectionPenBottom)
            painter.drawRect(self._rect)

            # painter.setPen(self.selectionPenTop)
            # painter.drawRect(self._rect)

            ########################################################################
            # !!! simdilik iptal, gorsel fazlalik olusturmakta !!!
            ########################################################################
            # if not self.isPinned and self.isActiveItem:
            #     painter.setPen(self.handlePen)
            #     painter.drawRect(self.topLeftHandle)
            #     painter.drawRect(self.topRightHandle)
            #     painter.drawRect(self.bottomRightHandle)
            #     painter.drawRect(self.bottomLeftHandle)
            ########################################################################

        # if option.state & QStyle.StateFlag.State_MouseOver:
        #     painter.setBrush(Qt.BrushStyle.NoBrush)
        #     painter.setPen(self.selectionPenBottom)
        #     painter.drawRect(self._rect)

        # font = painter.font()
        # font.setPointSize(self.fontPointSize)
        # painter.setFont(font)

        # # karenin altina yazsin yaziyi amcli ama bounding bozu degistirmek lazim.
        # # klasin simdilik.
        # # metrics.size() # this is metrics.boundingRect() size.
        # rect = metrics.boundingRect(self.text)
        # rect.moveTop(brect.bottom())
        # painter.drawText(rect, Qt.AlignCenter, self.text)

        # # # # # # debug start - pos() # # # # #
        # p = self.pos()
        # s = self.scenePos()
        # painter.drawText(self._rect, "{0:.2f},  {1:.2f}\n{2:.2f},  {3:.2f}".format(p.x(), p.y(), s.x(), s.y()))
        # # t = self.transformOriginPoint()
        # # painter.drawRect(t.x()-12, t.y()-12,24,24)
        # mapped = self.mapToScene(self._rect.topLeft())
        # painter.drawText(self._rect.x(), self._rect.y(), "{0:.2f}  {1:.2f}".format(mapped.x(), mapped.y()))
        # r = self.textItem.boundingRect()
        # r = self.mapRectFromItem(self.textItem, r)
        # painter.drawRect(r)
        # painter.drawText(self._rect.center(), "{0:f}  {1:f}".format(self.sceneWidth(), self.sceneHeight()))
        # # # # # # debug end - pos() # # # # #

    # ---------------------------------------------------------------------
    def html_dive_cevir(self, html_klasor_kayit_adres, dosya_kopyalaniyor_mu):

        buffer = QBuffer()
        buffer.open(QIODevice.OpenModeFlag.WriteOnly)
        self.ikonPixmap.save(buffer, "PNG")

        ikon_base64_buffer = buffer.data().toBase64()
        # ikon_base64_buffer = buffer.data().toBase64(QByteArray.Base64UrlEncoding)
        ikon_str = ikon_base64_buffer.data().decode("ascii")  # "utf-8" olmaz burda, binary.
        # print(ikon_str)

        # resim_adres= f'<img src="{item.filePathForSave}" style="object-fit: fill;"></img>'
        # img_str = f'<img src="{self.filePathForSave}" style="width:100%; height:100%;"></img>'

        dosya_adi = os.path.basename(self.filePathForSave)
        dosya_adres = self.filePathForSave
        if not html_klasor_kayit_adres:  # def dosyasi icine kaydet
            if self.isEmbeded:
                dosya_adres = os.path.join("files", dosya_adi)
        else:  # dosya html olarak baska bir yere kaydediliyor
            # kopyalanmazsa da, zaten embed olmayan dosya normal hddeki adresten yuklenecektir.
            if dosya_kopyalaniyor_mu:
                if not self.isEmbeded:  # embed ise zaten tmp klasorden hedef klasore baska metodta kopylanıyor hepsi.
                    dosya_adres = os.path.join(html_klasor_kayit_adres, "files", dosya_adi)

        a_str = f'<a href="{dosya_adres}">{dosya_adi}</a>'

        x = self.scenePos().x()
        y = self.scenePos().y()
        xs = self.scene().sceneRect().x()
        ys = self.scene().sceneRect().y()
        x = x - xs
        y = y - ys

        bicimSozluk = self.ver_karakter_bicimi()
        bold = "font-weight:bold;" if bicimSozluk["b"] else ""
        italic = "font-style:italic;" if bicimSozluk["i"] else ""
        underline = "underline" if bicimSozluk["u"] else ""
        strikeOut = "line-through" if bicimSozluk["s"] else ""
        overline = "overline" if bicimSozluk["o"] else ""
        bicimler1 = bold + italic
        if any((underline, strikeOut, overline)):
            bicimler2 = f"text-decoration: {underline} {strikeOut} {overline};"
        else:
            bicimler2 = ""

        renk_arkaPlan = f"({self.arkaPlanRengi.red()},{self.arkaPlanRengi.green()},{self.arkaPlanRengi.blue()},{self.arkaPlanRengi.alpha() / 255})"
        renk_yazi = f"({self.yaziRengi.red()},{self.yaziRengi.green()},{self.yaziRengi.blue()},{self.yaziRengi.alpha() / 255})"

        div_str = f"""
                    <div style="
                     background:rgba{renk_arkaPlan};
                     background-image:url('data:image/png;base64,{ikon_str}');
                     background-repeat:no-repeat;
                     background-position:center;
                     color:rgba{renk_yazi};
                     font-size:{self.fontPointSizeF()}pt; 
                     font-family:{self.font().family()};
                     {bicimler1}
                     {bicimler2}
                     position:absolute;
                     z-index:{int(self.zValue() * 10) if self.zValue() else 0};
                     width:{self.sceneWidth()}px;
                     height:{self.sceneHeight()}px;
                     top:{y}px;
                     left:{x}px;" id="{self._kim}">{a_str}</div>\n"""

        # /*background-image:url('{resim_adres}');*/
        return div_str



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__author__ = 'Erdinç Yılmaz'
__date__ = '3/28/16'

from PySide6.QtWidgets import QTabWidget
from PySide6.QtCore import Slot
from .treeModel import DokumanAgacModel


#######################################################################
class TabWidget(QTabWidget):

    # tabWidgetFocusedIn = Signal()

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):

        super(TabWidget, self).__init__(parent)
        self.modeller = []
        self.cModel = None

        self.setContentsMargins(0, 0, 0, 0)

    # # ---------------------------------------------------------------------
    # def mousePressEvent(self, a0):
    #     super(TabWidget, self).mousePressEvent(a0)
    #     print(self.cDokuman.sayfalar)
    #

    # ---------------------------------------------------------------------
    def model_ekle(self, tempDirPath):
        model = DokumanAgacModel(tempDirPath, self)
        # currentIndex yoksa -1 donduruyor o da ilk eklerken 0 yapar
        self.modeller.insert(self.currentIndex() + 1, model)
        self.cModel = model
        return model

    # ---------------------------------------------------------------------
    def model_sil(self, model):
        # idx = self.dokumanlar.index(dokuman)
        # for sayfa in dokuman.sayfalar:
        #     dokuman.sayfa_sil(sayfa)
        self.modeller.remove(model)
        model.sifirla()
        model.setParent(None)
        model.deleteLater()
        # TODO: ??
        # self.cDokuman = self.dokumanlar[idx]
        # self.cDokuman daha sonra set ediliyor, close_selected_tab tan _remove_tab cagriliyor
        # burasi _remove_tabtan cagriliyor, burdan cikinca ve _remove_tabtab cikinca
        # focus_current_tab_and_change_window_name cagriliyor orda tabWidget.cDokuman set ediliyor

    # ---------------------------------------------------------------------
    @Slot(int, int)
    def model_sira_degistir(self, frm, to):
        # tab order in tabWidget.tablar is important
        self.modeller[frm], self.modeller[to] = self.modeller[to], self.modeller[frm]

    # # ---------------------------------------------------------------------
    # def dokumana_sayfa_ekle(self, dokuman, sayfa):
    #     dokuman.sayfa_ekle(sayfa)
    #
    # # ---------------------------------------------------------------------
    # def dokumandan_sayfa_sil(self,dokuman, sayfa):
    #     dokuman.sayfa_sil(sayfa)

    # ---------------------------------------------------------------------
    def sayfalari_getir(self, widget):
        pass

    # ---------------------------------------------------------------------
    def current_sayfa(self):
        pass

    # # ---------------------------------------------------------------------
    # def tabInserted(self, p_int):
    #     print("eklendi ", p_int)
    #     super(TabWidget, self).tabInserted(p_int)
    #
    # # ---------------------------------------------------------------------
    # def tabRemoved(self, p_int):
    #     print("silindi", p_int)
    #     super(TabWidget, self).tabRemoved(p_int)

    # ---------------------------------------------------------------------
    def tab_olustur(self, view_widget, dosya_adi, sayfa_adi):

        idx = self.currentIndex()
        # self.insertTab((idx + 1), view, scene.fileName)

        self.insertTab((idx + 1), view_widget, "{} - {}".format(dosya_adi, sayfa_adi))
        self.setCurrentIndex(idx + 1)

        self.setTabToolTip(self.currentIndex(), self.tr("Unsaved File"))

    # ---------------------------------------------------------------------
    def get_all_widgets_temp_dir_paths(self):
        temp_dir_paths = []
        for i in range(self.count()):
            temp_dir_paths.append(self.widget(i).scene().tempDirPath)
        return temp_dir_paths

    # # ---------------------------------------------------------------------
    # def get_index_of_tab_from_path__(self, path):
    #     index = None
    #     for i in range(self.count()):
    #         tab = self.widget(i)
    #         if tab.scene().saveFilePath == path:
    #             index = self.indexOf(tab)
    #             break
    #     return index

    # ---------------------------------------------------------------------
    def get_index_of_tab_from_path(self, path):
        index = None
        for model in self.modeller:
            if model.saveFilePath == path:
                index = self.modeller.index(model)
                break
        return index

    # # ---------------------------------------------------------------------
    # def get_all_paths_of_tabs(self):
    #     paths = []
    #     for i in range(self.count()):
    #         paths.append(self.widget(i).path)
    #     return paths

    # ---------------------------------------------------------------------
    def get_current_widget_with_path(self, path):

        index = self.get_index_of_tab_from_path(path)
        # if index and index > -1:  # index 0 olabiliyor o da if 0 olunca problem oluyor :)
        if index is not None and index > -1:  # if index or index==0 : if index > -1 gibi bir sey de belki olurdu:
            return self.widget(index)
        else:
            return None

    # ---------------------------------------------------------------------
    def set_current_widget_with_path(self, path):

        currentWidget = self.get_current_widget_with_path(path)
        if currentWidget:
            self.setCurrentWidget(currentWidget)
            return True
        else:
            return False



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '16/11/22'
__author__ = 'Erdinç Yılmaz'

from PySide6.QtWidgets import QHBoxLayout, QLineEdit, QPushButton, QTextEdit, QWidget, QVBoxLayout
from ..yw.yuzenWidget import YuzenWidget


#######################################################################
class TercumeYW(YuzenWidget):

    # ---------------------------------------------------------------------
    def __init__(self, parent=None):
        super(TercumeYW, self).__init__(kapatilabilir_mi=True, parent=parent)

        self.okuBtn = QPushButton(self.tr("Oku"), self)
        self.girisYaziLE = QLineEdit(self)
        self.tercumeTrBtn = QPushButton(self.tr("Tr"), self)
        self.tercumeTrBtn.clicked.connect(self.yap_tr_tercume)
        self.tercumeEngBtn = QPushButton(self.tr("Eng"), self)
        self.tercumeEngBtn.clicked.connect(self.yap_eng_tercume)

        layTercumeBtn = QHBoxLayout()
        layTercumeBtn.addWidget(self.tercumeTrBtn)
        layTercumeBtn.addWidget(self.tercumeEngBtn)

        self.sonucTE = QTextEdit(self)
        self.ekleWidget(self.sonucTE)

        temelW = QWidget(self)
        anaLay = QVBoxLayout(temelW)
        anaLay.addWidget(self.okuBtn)
        anaLay.addWidget(self.girisYaziLE)
        anaLay.addLayout(layTercumeBtn)
        anaLay.addWidget(self.sonucTE)
        anaLay.addStretch()

        self.ekleWidget(temelW)

        self.setMinimumWidth(250)
        self.setMinimumHeight(170)

    # ---------------------------------------------------------------------
    def yap_tr_tercume(self):
        sonuc = "tr"
        self.sonucTE.setText(sonuc)

    # ---------------------------------------------------------------------
    def yap_eng_tercume(self):
        sonuc = "eng"
        self.sonucTE.setText(sonuc)

    # # ---------------------------------------------------------------------
    # def olustur_radio(self):
    #
    #
    #     self.anaLay.addLayout(radioLay)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '05/Nov/2018'
__author__ = 'Erdinç Yılmaz'

from .undoRedoSiniflar import (UndoableSayfaAdiDegistir, UndoableAddItem, UndoableRemoveItem, UndoableGroup,
                                    UndoableUnGroup, UndoableParent, UndoableUnParent, UndoableResizeBaseItem, UndoableScaleBaseItemByResizing,
                                    UndoableScalePathItemByScalingPath, UndoableSetFont, UndoableSetFontSizeF, UndoableRotate,
                                    UndoableRotateWithOffset, UndoableSetZValue, UndoableSetLineStyle, UndoableSetLineJoinStyle,
                                    UndoableSetLineCapStyle, UndoableSetLineColor, UndoableSetTextColor,
                                    UndoableSetLineColorAlpha, UndoableSetTextColorAlpha, UndoableSetItemBackgroundColor,
                                    UndoableSetItemBackgroundColorAlpha, UndoableStilAdiDegistir, UndoableStiliNesneyeUygula, UndoableStiliUygula,
                                    UndoableSetImageOpacity, UndoableSetSceneBackgroundBrush, UndoableSetSceneBackgroundImage,
                                    UndoableEmbedSceneBackgroundImage, UndoableSetPinStatus, UndoableItemSetText, UndoableItemCustomCommand,
                                    UndoableEmbedImage, UndoableEmbedVideo, UndoableMove, UndoableEmbedFile, UndoableResizeLineItem,
                                    UndoableMovePathPoint, UndoableSetTextAlignment, UndoableSetCharacterFormat, UndoableSetLineWidthF,
                                    UndoableConvertToPlainText, UndoRedoBaglantisiYaziNesnesiDocuna, UndoableScaleLineItemByScalingLine,
                                    UndoableScaleTextItemByResizing, UndoableScaleGroupItemByResizing, UndoableDeletePathPoint)


# ---------------------------------------------------------------------
def undoRedoBaglantisiYaziNesnesiDocuna(undoStack, description, doc):
    command = UndoRedoBaglantisiYaziNesnesiDocuna(description, doc)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSayfaAdiDegistir(undoStack, description, sayfa, sayfa_eski_adi):
    command = UndoableSayfaAdiDegistir(description, sayfa, sayfa_eski_adi)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableAddItem(undoStack, description, scene, item):
    command = UndoableAddItem(description, scene, item)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableRemoveItem(undoStack, description, scene, item, addToUnGroupedRootItems):
    command = UndoableRemoveItem(description, scene, item, addToUnGroupedRootItems)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableGroup(undoStack, description, items, scene):
    command = UndoableGroup(description, items, scene)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableUnGroup(undoStack, description, group):
    command = UndoableUnGroup(description, group)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableParent(undoStack, description, item, parentItem, yeniPos):
    command = UndoableParent(description, item, parentItem, yeniPos)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableUnParent(undoStack, description, item, yeniParentItem, yeniPos):
    command = UndoableUnParent(description, item, yeniParentItem, yeniPos)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableResizeBaseItem(undoStack, description, item, yeniRect, eskiRect, eskiPos):
    command = UndoableResizeBaseItem(description, item, yeniRect, eskiRect, eskiPos)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableResizeLineItem(undoStack, description, item, yeniLine, eskiLine, eskiPos, degisenNokta=None):
    command = UndoableResizeLineItem(description, item, yeniLine, eskiLine, eskiPos, degisenNokta)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableScaleGroupItemByResizing(undoStack, description, item, yeniRect, scaleFactor, yeniPos):
    command = UndoableScaleGroupItemByResizing(description, item, yeniRect, scaleFactor, yeniPos)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableScaleBaseItemByResizing(undoStack, description, item, yeniRect, scaleFactor, yeniPos):
    command = UndoableScaleBaseItemByResizing(description, item, yeniRect, scaleFactor, yeniPos)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableScaleTextItemByResizing(undoStack, description, item, scaleFactor, fontPointSizeF):
    command = UndoableScaleTextItemByResizing(description, item, scaleFactor, fontPointSizeF)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableScalePathItemByScalingPath(undoStack, description, item, path, scaleFactor):
    command = UndoableScalePathItemByScalingPath(description, item, path, scaleFactor)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableScaleLineItemByScalingLine(undoStack, description, item, line, scaleFactor):
    command = UndoableScaleLineItemByScalingLine(description, item, line, scaleFactor)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetFont(undoStack, description, item, font):
    command = UndoableSetFont(description, item, font)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetFontSizeF(undoStack, description, item, fontPointSizeF):
    command = UndoableSetFontSizeF(description, item, fontPointSizeF)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetCharacterFormat(undoStack, description, item, sozluk):
    command = UndoableSetCharacterFormat(description, item, sozluk)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetTextAlignment(undoStack, description, item, hiza):
    command = UndoableSetTextAlignment(description, item, hiza)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableRotate(undoStack, description, item, rotation):
    command = UndoableRotate(description, item, rotation)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableRotateWithOffset(undoStack, description, item, rotation):
    command = UndoableRotateWithOffset(description, item, rotation)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetZValue(undoStack, description, item, zValue):
    command = UndoableSetZValue(description, item, zValue)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineStyle(undoStack, description, item, lineStyle):
    command = UndoableSetLineStyle(description, item, lineStyle)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineJoinStyle(undoStack, description, item, joinStyle):
    command = UndoableSetLineJoinStyle(description, item, joinStyle)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineCapStyle(undoStack, description, item, capStyle):
    command = UndoableSetLineCapStyle(description, item, capStyle)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineWidthF(undoStack, description, item, widthF):
    command = UndoableSetLineWidthF(description, item, widthF)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineColor(undoStack, description, item, color, renkSecicidenMi):
    command = UndoableSetLineColor(description, item, color, renkSecicidenMi)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetTextColor(undoStack, description, item, color, renkSecicidenMi):
    command = UndoableSetTextColor(description, item, color, renkSecicidenMi)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetItemBackgroundColor(undoStack, description, item, color, renkSecicidenMi):
    command = UndoableSetItemBackgroundColor(description, item, color, renkSecicidenMi)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetItemBackgroundColorAlpha(undoStack, description, item, color):
    command = UndoableSetItemBackgroundColorAlpha(description, item, color)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetLineColorAlpha(undoStack, description, item, color):
    command = UndoableSetLineColorAlpha(description, item, color)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetTextColorAlpha(undoStack, description, item, color):
    command = UndoableSetTextColorAlpha(description, item, color)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableStilAdiDegistir(undoStack, description, nesne, nesne2, yeni_isim):
    command = UndoableStilAdiDegistir(description, nesne, nesne2, yeni_isim)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableStiliNesneyeUygula(undoStack, description, item, pen, brush, font, yaziRengi, cizgiRengi):
    command = UndoableStiliNesneyeUygula(description, item, pen, brush, font, yaziRengi, cizgiRengi)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableStiliUygula(undoStack, description, eskiPen, yeniPen, yaziRengi, eskiYaziRengi, cizgiRengi,
                        eskiCizgiRengi,
                        eskiArkaPlanRengi, yeniArkaPlanRengi, eskiFont, yeniFont, scene):
    command = UndoableStiliUygula(description, eskiPen, yeniPen, yaziRengi, eskiYaziRengi, cizgiRengi,
                                  eskiCizgiRengi,
                                  eskiArkaPlanRengi, yeniArkaPlanRengi, eskiFont, yeniFont,
                                  scene)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetImageOpacity(undoStack, description, item, imageOpacity):
    command = UndoableSetImageOpacity(description, item, imageOpacity)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetSceneBackgroundBrush(undoStack, description, view, color):
    command = UndoableSetSceneBackgroundBrush(description, view, color)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetSceneBackgroundImage(undoStack, description, view, imagePath):
    command = UndoableSetSceneBackgroundImage(description, view, imagePath)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableEmbedSceneBackgroundImage(undoStack, description, view, ):
    command = UndoableEmbedSceneBackgroundImage(description, view)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableSetPinStatus(undoStack, description, item, value):
    command = UndoableSetPinStatus(description, item, value)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableItemSetText(undoStack, description, item, eskiText, text):
    command = UndoableItemSetText(description, item, eskiText, text)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableConvertToPlainText(undoStack, description, item):
    command = UndoableConvertToPlainText(description, item)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableItemCustomCommand(undoStack, description, item, command):
    command = UndoableItemCustomCommand(description, item, command)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableEmbedImage(undoStack, description, item):
    command = UndoableEmbedImage(description, item)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableEmbedVideo(undoStack, description, item):
    command = UndoableEmbedVideo(description, item)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableEmbedFile(undoStack, description, item):
    command = UndoableEmbedFile(description, item)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableMove(undoStack, description, movedItem, eskiPosition):
    command = UndoableMove(description, movedItem, eskiPosition)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableMovePathPoint(undoStack, description, item, movedPointIndex, eskiPosTuple, yeniPosTuple):
    command = UndoableMovePathPoint(description, item, movedPointIndex, eskiPosTuple, yeniPosTuple)
    undoStack.push(command)


# ---------------------------------------------------------------------
def undoableDeletePathPoint(undoStack, description, item, eskiPath, yeniPath):
    command = UndoableDeletePathPoint(description, item, eskiPath, yeniPath)
    undoStack.push(command)



# -*- coding: utf-8 -*-
# .

__project_name__ = 'Defter'
__date__ = '03/Apr/2017'
__author__ = 'Erdinç Yılmaz'

from html.parser import HTMLParser


########################################################################
class ImgSrcParser(HTMLParser):

    def __init__(self):
        super(ImgSrcParser, self).__init__()
        # HTMLParser.HTMLParser.__init__(self)
        # self.data = []
        self.urls = []

    def handle_starttag(self, tag, attrs):
        if tag == "img":
            # attrs is a list of tuples, (attr , val)
            self.urls.append(dict(attrs)["src"])

    # def handle_data(self, data):
    #     self.data.append(data)

    def error(self, message):
        pass
